#!/usr/bin/env python
#
# Copyright 2013 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#


import random
import numpy

import pmt
from gnuradio import gr, gr_unittest
from gnuradio import digital
from gnuradio import blocks
from gnuradio import channels
from gnuradio.digital.ofdm_txrx import ofdm_tx, ofdm_rx
from gnuradio.digital.utils import tagged_streams

# Set this to true if you need to write out data
LOG_DEBUG_INFO = False


class ofdm_tx_fg (gr.top_block):
    def __init__(
            self,
            data,
            len_tag_key,
            scramble_bits=False,
            additional_tags=[]):
        gr.top_block.__init__(self, "ofdm_tx")
        tx_data, tags = tagged_streams.packets_to_vectors((data,), len_tag_key)
        src = blocks.vector_source_b(data, False, 1, tags + additional_tags)
        self.tx = ofdm_tx(
            packet_length_tag_key=len_tag_key,
            debug_log=LOG_DEBUG_INFO,
            scramble_bits=scramble_bits)
        self.sink = blocks.vector_sink_c()
        self.connect(src, self.tx, self.sink)

    def get_tx_samples(self):
        return self.sink.data()


class ofdm_rx_fg (gr.top_block):
    def __init__(
            self,
            samples,
            len_tag_key,
            channel=None,
            prepend_zeros=100,
            scramble_bits=False):
        gr.top_block.__init__(self, "ofdm_rx")
        if prepend_zeros:
            samples = (0,) * prepend_zeros + tuple(samples)
        src = blocks.vector_source_c(list(samples) + [0, ] * 1000)
        self.rx = ofdm_rx(
            frame_length_tag_key=len_tag_key,
            debug_log=LOG_DEBUG_INFO,
            scramble_bits=scramble_bits)
        if channel is not None:
            self.connect(src, channel, self.rx)
        else:
            self.connect(src, self.rx)
        self.sink = blocks.vector_sink_b()
        self.connect(self.rx, self.sink)

    def get_rx_bytes(self):
        return self.sink.data()


class test_ofdm_txrx (gr_unittest.TestCase):

    def setUp(self):
        random.seed(0)
        self.tb = gr.top_block()

    def tearDown(self):
        self.tb = None

    def test_001_tx(self):
        """ Just make sure the Tx works in general """
        # This tag gets put onto the first item of the transmit data,
        # it should be transmitted first, too
        timing_tag = gr.tag_t()
        timing_tag.offset = 0
        timing_tag.key = pmt.string_to_symbol('tx_timing')
        timing_tag.value = pmt.to_pmt('now')
        len_tag_key = 'frame_len'
        n_bytes = 52
        n_samples_expected = (numpy.ceil(1.0 * (n_bytes + 4) / 6) + 3) * 80
        test_data = [random.randint(0, 255) for x in range(n_bytes)]
        tx_fg = ofdm_tx_fg(
            test_data,
            len_tag_key,
            additional_tags=[
                timing_tag,
            ])
        tx_fg.run()
        self.assertEqual(len(tx_fg.get_tx_samples()), n_samples_expected)
        tags_rx = [gr.tag_to_python(x) for x in tx_fg.sink.tags()]
        tags_rx = sorted([(x.offset, x.key, x.value) for x in tags_rx])
        tags_expected = [
            (0, 'frame_len', n_samples_expected),
            (0, 'tx_timing', 'now'),
        ]
        self.assertEqual(tags_rx, tags_expected)

    def test_002_rx_only_noise(self):
        """ Run the RX with only noise, check it doesn't crash
        or return a burst. """
        len_tag_key = 'frame_len'
        samples = (0,) * 1000
        channel = channels.channel_model(0.1)
        rx_fg = ofdm_rx_fg(samples, len_tag_key, channel)
        rx_fg.run()
        self.assertEqual(len(rx_fg.get_rx_bytes()), 0)

    def test_003_tx1packet(self):
        """ Transmit one packet, with slight AWGN and slight frequency + timing offset.
        Check packet is received and no bit errors have occurred. """
        len_tag_key = 'frame_len'
        n_bytes = 21
        fft_len = 64
        test_data = list([random.randint(0, 255) for x in range(n_bytes)])
        # 1.0/fft_len is one sub-carrier, a fine freq offset stays below that
        freq_offset = 1.0 / fft_len * 0.7
        #channel = channels.channel_model(0.01, freq_offset)
        channel = None
        # Tx
        tx_fg = ofdm_tx_fg(test_data, len_tag_key)
        tx_fg.run()
        tx_samples = tx_fg.get_tx_samples()
        # Rx
        rx_fg = ofdm_rx_fg(tx_samples, len_tag_key, channel, prepend_zeros=100)
        rx_fg.run()
        rx_data = rx_fg.get_rx_bytes()
        self.assertEqual(list(tx_fg.tx.sync_word1), list(rx_fg.rx.sync_word1))
        self.assertEqual(list(tx_fg.tx.sync_word2), list(rx_fg.rx.sync_word2))
        self.assertEqual(test_data, rx_data)

    def test_003_tx1packet_scramble(self):
        """ Same as before, use scrambler. """
        len_tag_key = 'frame_len'
        n_bytes = 21
        fft_len = 64
        test_data = list([random.randint(0, 255) for x in range(n_bytes)])
        # 1.0/fft_len is one sub-carrier, a fine freq offset stays below that
        freq_offset = 1.0 / fft_len * 0.7
        #channel = channels.channel_model(0.01, freq_offset)
        channel = None
        # Tx
        tx_fg = ofdm_tx_fg(test_data, len_tag_key, scramble_bits=True)
        tx_fg.run()
        tx_samples = tx_fg.get_tx_samples()
        # Rx
        rx_fg = ofdm_rx_fg(
            tx_samples,
            len_tag_key,
            channel,
            prepend_zeros=100,
            scramble_bits=True)
        rx_fg.run()
        rx_data = rx_fg.get_rx_bytes()
        self.assertEqual(list(tx_fg.tx.sync_word1), list(rx_fg.rx.sync_word1))
        self.assertEqual(list(tx_fg.tx.sync_word2), list(rx_fg.rx.sync_word2))
        self.assertEqual(test_data, rx_data)

    def test_004_tx1packet_large_fO(self):
        """ Transmit one packet, with slight AWGN and large frequency offset.
        Check packet is received and no bit errors have occurred. """
        fft_len = 64
        len_tag_key = 'frame_len'
        n_bytes = 21
        test_data = list([random.randint(0, 255) for x in range(n_bytes)])
        #test_data = tuple([255 for x in range(n_bytes)])
        # 1.0/fft_len is one sub-carrier
        frequency_offset = 1.0 / fft_len * 2.5
        channel = channels.channel_model(0.00001, frequency_offset)
        # Tx
        tx_fg = ofdm_tx_fg(test_data, len_tag_key)
        tx_fg.run()
        tx_samples = tx_fg.get_tx_samples()
        # Rx
        rx_fg = ofdm_rx_fg(tx_samples, len_tag_key, channel, prepend_zeros=100)
        rx_fg.run()
        rx_data = rx_fg.get_rx_bytes()
        self.assertEqual(test_data, rx_data)


if __name__ == '__main__':
    gr_unittest.run(test_ofdm_txrx)



import collections
import numbers

from grc.core.utils.descriptors import Evaluated, EvaluatedEnum, EvaluatedPInt


class A(object):
    def __init__(self, **kwargs):
        self.called = collections.defaultdict(int)
        self.errors = []
        self.namespace = kwargs

    def add_error_message(self, msg):
        self.errors.append(msg)

    @property
    def parent_block(self):
        return self

    def evaluate(self, expr):
        self.called['evaluate'] += 1
        return eval(expr, self.namespace)

    @Evaluated(int, 1)
    def foo(self):
        self.called['foo'] += 1
        return eval(self._foo)

    bar = Evaluated(numbers.Real, 1.0, name='bar')

    test = EvaluatedEnum(['a', 'b'], 'a', name='test')

    lala = EvaluatedPInt()


def test_fixed_value():
    a = A()
    a.foo = 10

    assert not hasattr(a, '_foo')
    assert a.foo == 10
    assert a.called['foo'] == 0
    delattr(a, 'foo')
    assert a.foo == 10
    assert a.called['foo'] == 0


def test_evaled():
    a = A()
    a.foo = '${ 10 + 1 }'
    assert getattr(a, '_foo') == '10 + 1'
    assert a.foo == 11 and a.foo == 11
    assert a.called['foo'] == 1
    assert a.called['evaluate'] == 0
    delattr(a, 'foo')
    assert a.foo == 11 and a.foo == 11
    assert a.called['foo'] == 2
    assert not a.errors
    a.foo = u'${ 10 + 2 }'
    assert a.foo == 12


def test_evaled_with_default():
    a = A()
    a.bar = '${ 10 + 1 }'
    assert getattr(a, '_bar') == '10 + 1'
    assert a.bar == 11.0 and type(a.bar) == int
    assert a.called['evaluate'] == 1
    assert not a.errors


def test_evaled_int_with_default():
    a = A(ll=10)
    a.lala = '${ ll * 2 }'
    assert a.lala == 20
    a.namespace['ll'] = -10
    assert a.lala == 20
    del a.lala
    assert a.lala == 1
    assert not a.errors


def test_evaled_enum_fixed_value():
    a = A()
    a.test = 'a'
    assert not hasattr(a, '_test')
    assert a.test == 'a' and type(a.test) == str
    assert not a.errors


def test_evaled_enum():
    a = A(bla=False)
    a.test = '${ "a" if bla else "b" }'
    assert a.test == 'b'
    a.namespace['bla'] = True
    assert a.test == 'b'
    del a.test
    assert a.test == 'a'
    assert not a.errors


def test_class_access():
    a = A()
    a.foo = '${ meme }'
    descriptor = getattr(a.__class__, 'foo')
    assert descriptor.name_raw == '_foo'



"""
Copyright 2021 Ettus Research, A National Instruments Brand.

SPDX-License-Identifier: GPL-3.0-or-later
"""

import sys

MAX_NUM_CHANNELS = 16

MAIN_TMPL = """\
id: uhd_rfnoc_siggen
label: RFNoC SigGen Block
category: '[Core]/UHD/RFNoC/Blocks'

parameters:
- id: num_chans
  label: Channel count
  dtype: int
  options: [ ${", ".join([str(n) for n in range(1, max_num_chans+1)])} ]
  default: 1
  hide: ${'$'}{ 'part' }
- id: block_args
  label: Block Args
  dtype: string
  default: ""
  hide: ${'$'}{ 'part' if not block_args else 'none'}
- id: device_select
  label: Device Select
  dtype: int
  default: -1
  hide: ${'$'}{ 'part' if device_select == -1 else 'none'}
- id: instance_index
  label: Instance Select
  dtype: int
  default: -1
  hide: ${'$'}{ 'part' if instance_index == -1 else 'none'}
${siggen_params}
outputs:
- domain: rfnoc
  dtype: 'sc16'
  multiplicity: ${'$'}{num_chans}

templates:
  imports: |-
    from gnuradio import uhd
  make: |-
    uhd.rfnoc_siggen(
        self.rfnoc_graph,
        uhd.device_addr(${'$'}{block_args}),
        ${'$'}{device_select},
        ${'$'}{instance_index})
${init_params}  callbacks:
${callback_params}

documentation: |-
  RFNoC SigGen Block:
  A simple signal generator that does not require an input data steam to
  generate output.

  Channel count:
  Number of channels / streams to use with the RFNoC SigGen block. Note,
  this is defined by the RFNoC SigGen Block's FPGA build parameters and
  GNU Radio Companion is not aware of this value. An error will occur at
  runtime when connecting blocks if the number of channels is too large.

file_format: 1
"""

SIGGEN_PARAM = """- id: signal_type${n}
  label: 'Ch${n}: Signal Type'
  dtype: enum
  options: ['NOISE', 'CONSTANT', 'SINE_WAVE']
  default: 'NOISE'
  hide: ${'$'}{ 'part' if num_chans > ${n} else 'all'}
- id: sample_rate${n}
  label: 'Ch${n}: Sample Rate'
  dtype: float
  default: samp_rate
  hide: ${'$'}{'part' if num_chans > ${n} else 'all'}
- id: enable${n}
  label: 'Ch${n}: Enable'
  dtype: bool
  default: true
  hide: ${'$'}{'part' if num_chans > ${n} else 'all'}
- id: signal_frequency${n}
  label: 'Ch${n}: Frequency'
  dtype: float
  default: 0.0
  hide: ${'$'}{ 'none' if (signal_type${n} == 'SINE_WAVE') and num_chans > ${n} else 'all' }
- id: signal_amplitude${n}
  label: 'Ch${n}: Amplitude'
  dtype: float
  default: 1.0
  hide: ${'$'}{ 'none' if num_chans > ${n} else 'all' }
"""

INIT_PARAM = """    ${'%'} if signal_type${n} == 'NOISE' and context.get('num_chans')() > ${n}:
    self.${'$'}{id}.set_amplitude(${'$'}{signal_amplitude${n}}, ${n})
    ${'%'} endif
    ${'%'} if signal_type${n} == 'SINE_WAVE' and context.get('num_chans')() > ${n}:
    self.${'$'}{id}.set_sine_frequency(${'$'}{signal_frequency${n}}, ${'$'}{sample_rate${n}}, ${n})
    self.${'$'}{id}.set_amplitude(${'$'}{signal_amplitude${n}}, ${n})
    ${'%'} endif
    ${'%'} if signal_type${n} == 'CONSTANT' and context.get('num_chans')() > ${n}:
    self.${'$'}{id}.set_constant(${'$'}{signal_amplitude${n}}, ${n})
    ${'%'} endif
    ${'%'} if context.get('num_chans')() > ${n}:
    self.${'$'}{id}.set_waveform('${'$'}{signal_type${n}}', ${n})
    self.${'$'}{id}.set_enable(${'$'}{enable${n}}, ${n})
    ${'%'} endif
"""

CALLBACKS_PARAM = """  - |
    ${'%'} if signal_type${n} == 'NOISE' and context.get('num_chans')() > ${n}:
    self.${'$'}{id}.set_amplitude(${'$'}{signal_amplitude${n}}, ${n})
    ${'%'} endif
  - |
    ${'%'} if signal_type${n} == 'SINE_WAVE' and context.get('num_chans')() > ${n}:
    self.${'$'}{id}.set_sine_frequency(${'$'}{signal_frequency${n}}, ${'$'}{sample_rate${n}}, ${n})
    self.${'$'}{id}.set_amplitude(${'$'}{signal_amplitude${n}}, ${n})
    ${'%'} endif
  - |
    ${'%'} if signal_type${n} == 'CONSTANT' and context.get('num_chans')() > ${n}:
    self.${'$'}{id}.set_constant(${'$'}{signal_amplitude${n}}, ${n})
    ${'%'} endif
  - |
    ${'%'} if context.get('num_chans')() > ${n}:
    self.${'$'}{id}.set_waveform('${'$'}{signal_type${n}}', ${n})
    self.${'$'}{id}.set_enable(${'$'}{enable${n}}, ${n})
    ${'%'} endif

"""


def parse_tmpl(_tmpl, **kwargs):
    """ Render _tmpl using the kwargs """
    from mako.template import Template

    block_template = Template(_tmpl)
    return str(block_template.render(**kwargs))


if __name__ == '__main__':
    file = sys.argv[1]
    siggen_params = ''.join([
        parse_tmpl(SIGGEN_PARAM, n=n)
        for n in range(MAX_NUM_CHANNELS)
    ])
    init_params = ''.join([
        parse_tmpl(INIT_PARAM, n=n)
        for n in range(MAX_NUM_CHANNELS)
    ])
    callback_params = ''.join([
        parse_tmpl(CALLBACKS_PARAM, n=n)
        for n in range(MAX_NUM_CHANNELS)
    ])
    open(file, 'w').write(
        parse_tmpl(
            MAIN_TMPL,
            max_num_chans=MAX_NUM_CHANNELS,
            siggen_params=siggen_params,
            init_params=init_params,
            callback_params=callback_params
        )
    )



#!/usr/bin/env python
#
# Copyright 2008,2010,2012,2013 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#

from gnuradio import gr, gr_unittest, filter, blocks

import cmath
import math


def fir_filter(x, taps, decim=1):
    y = []
    x2 = (len(taps) - 1) * [0, ] + x
    for i in range(0, len(x), decim):
        yi = 0
        for j in range(len(taps)):
            yi += taps[len(taps) - 1 - j] * x2[i + j]
        y.append(yi)
    return y


def sig_source_s(samp_rate, freq, amp, N):
    t = [float(x) / samp_rate for x in range(N)]
    y = [int(100 * math.sin(2. * math.pi * freq * x)) for x in t]
    return y


def sig_source_c(samp_rate, freq, amp, N):
    t = [float(x) / samp_rate for x in range(N)]
    y = [math.cos(2. * math.pi * freq * x) +
         1j * math.sin(2. * math.pi * freq * x) for x in t]
    return y


def mix(lo, data):
    y = [lo_i * data_i for lo_i, data_i in zip(lo, data)]
    return y


class test_freq_xlating_filter(gr_unittest.TestCase):

    def setUp(self):
        self.tb = gr.top_block()

    def tearDown(self):
        self.tb = None

    def generate_ccf_source(self):
        self.fs = fs = 1
        self.fc = fc = 0.3
        self.bw = bw = 0.1
        self.taps = filter.firdes.low_pass(1, fs, bw, bw / 4)
        times = list(range(1024))
        self.src_data = [
            cmath.exp(-2j * cmath.pi * fc / fs * (t / 100.0)) for t in times]

    def generate_ccc_source(self):
        self.fs = fs = 1
        self.fc = fc = 0.3
        self.bw = bw = 0.1
        self.taps = filter.firdes.complex_band_pass(
            1, fs, -bw / 2, bw / 2, bw / 4)
        times = list(range(1024))
        self.src_data = [
            cmath.exp(-2j * cmath.pi * fc / fs * (t / 100.0)) for t in times]

    def assert_fft_ok(self, expected_result, result_data):
        expected_result = expected_result[:len(result_data)]
        self.assertComplexTuplesAlmostEqual(expected_result, result_data,
                                            places=5)

    def test_fft_filter_ccf_001(self):
        self.generate_ccf_source()

        decim = 1
        lo = sig_source_c(self.fs, -self.fc, 1, len(self.src_data))
        despun = mix(lo, self.src_data)
        expected_data = fir_filter(despun, self.taps, decim)

        src = blocks.vector_source_c(self.src_data)
        op = filter.freq_xlating_fft_filter_ccc(
            decim, self.taps, self.fc, self.fs)
        dst = blocks.vector_sink_c()
        self.tb.connect(src, op, dst)
        self.tb.run()
        result_data = dst.data()
        self.assert_fft_ok(expected_data, result_data)

    def test_fft_filter_ccf_002(self):
        self.generate_ccf_source()

        decim = 4
        lo = sig_source_c(self.fs, -self.fc, 1, len(self.src_data))
        despun = mix(lo, self.src_data)
        expected_data = fir_filter(despun, self.taps, decim)

        src = blocks.vector_source_c(self.src_data)
        op = filter.freq_xlating_fft_filter_ccc(
            decim, self.taps, self.fc, self.fs)
        dst = blocks.vector_sink_c()
        self.tb.connect(src, op, dst)
        self.tb.run()
        result_data = dst.data()
        self.assert_fft_ok(expected_data, result_data)

    def test_fft_filter_ccc_001(self):
        self.generate_ccc_source()

        decim = 1
        lo = sig_source_c(self.fs, -self.fc, 1, len(self.src_data))
        despun = mix(lo, self.src_data)
        expected_data = fir_filter(despun, self.taps, decim)

        src = blocks.vector_source_c(self.src_data)
        op = filter.freq_xlating_fft_filter_ccc(
            decim, self.taps, self.fc, self.fs)
        dst = blocks.vector_sink_c()
        self.tb.connect(src, op, dst)
        self.tb.run()
        result_data = dst.data()
        self.assert_fft_ok(expected_data, result_data)

    def test_fft_filter_ccc_002(self):
        self.generate_ccc_source()

        decim = 4
        lo = sig_source_c(self.fs, -self.fc, 1, len(self.src_data))
        despun = mix(lo, self.src_data)
        expected_data = fir_filter(despun, self.taps, decim)

        src = blocks.vector_source_c(self.src_data)
        op = filter.freq_xlating_fft_filter_ccc(
            decim, self.taps, self.fc, self.fs)
        dst = blocks.vector_sink_c()
        self.tb.connect(src, op, dst)
        self.tb.run()
        result_data = dst.data()
        self.assert_fft_ok(expected_data, result_data)


if __name__ == '__main__':
    gr_unittest.run(test_freq_xlating_filter)



#!/usr/bin/env python
#
# Copyright 2016 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#


from gnuradio import gr, gr_unittest, blocks


class test_block_behavior(gr_unittest.TestCase):

    def setUp(self):
        self.tb = gr.top_block()

    def tearDown(self):
        self.tb = None

    def test_000(self):
        '''
        Tests the max noutput sizes set by the scheduler. When creating
        the block, there is no block_detail and so the max buffer size
        is 0. When the top_block is run, it builds the detail and
        buffers and sets the max value. test_0001 tests when the
        max_noutput_items is set by hand.

        '''

        src = blocks.null_source(gr.sizeof_float)
        op = blocks.head(gr.sizeof_float, 100)
        snk = blocks.null_sink(gr.sizeof_float)

        maxn_pre = op.max_noutput_items()

        self.tb.connect(src, op, snk)
        self.tb.run()

        maxn_post = op.max_noutput_items()

        self.assertEqual(maxn_pre, 0)
        self.assertEqual(maxn_post, 16384)

    def test_001(self):
        '''
        Tests the max noutput size when being explicitly set.
        '''

        src = blocks.null_source(gr.sizeof_float)
        op = blocks.head(gr.sizeof_float, 100)
        snk = blocks.null_sink(gr.sizeof_float)

        op.set_max_noutput_items(1024)

        maxn_pre = op.max_noutput_items()

        self.tb.connect(src, op, snk)
        self.tb.run()

        maxn_post = op.max_noutput_items()

        self.assertEqual(maxn_pre, 1024)
        self.assertEqual(maxn_post, 1024)


if __name__ == '__main__':
    gr_unittest.run(test_block_behavior)



#!/usr/bin/env python
#
# Copyright 2011-2013 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#


import random
import math
import numpy

from gnuradio import gr, gr_unittest, digital, filter, blocks, analog


class test_fll_band_edge_cc(gr_unittest.TestCase):

    def setUp(self):
        random.seed(0)
        self.tb = gr.top_block()

    def tearDown(self):
        self.tb = None

    def test01(self):
        sps = 4
        rolloff = 0.35
        bw = 2 * math.pi / 100.0
        ntaps = 45

        # Create pulse shape filter
        rrc_taps = filter.firdes.root_raised_cosine(
            sps, sps, 1.0, rolloff, ntaps)

        # The frequency offset to correct
        foffset = 0.2 / (2.0 * math.pi)

        # Create a set of 1's and -1's, pulse shape and interpolate to sps
        random.seed(42)  # Fix seed
        data = [2.0 * random.randint(0, 2) - 1.0 for i in range(200)]
        self.src = blocks.vector_source_c(data, False)
        self.rrc = filter.interp_fir_filter_ccf(sps, rrc_taps)

        # Mix symbols with a complex sinusoid to spin them
        self.nco = analog.sig_source_c(1, analog.GR_SIN_WAVE, foffset, 1)
        self.mix = blocks.multiply_cc()

        # FLL will despin the symbols to an arbitrary phase
        self.fll = digital.fll_band_edge_cc(sps, rolloff, ntaps, bw)

        # Create sinks for all outputs of the FLL
        # we will only care about the freq and error outputs
        self.vsnk_frq = blocks.vector_sink_f()
        self.nsnk_fll = blocks.null_sink(gr.sizeof_gr_complex)
        self.nsnk_phs = blocks.null_sink(gr.sizeof_float)
        self.nsnk_err = blocks.null_sink(gr.sizeof_float)

        # Connect the blocks
        self.tb.connect(self.nco, (self.mix, 1))
        self.tb.connect(self.src, self.rrc, (self.mix, 0))
        self.tb.connect(self.mix, self.fll, self.nsnk_fll)
        self.tb.connect((self.fll, 1), self.vsnk_frq)
        self.tb.connect((self.fll, 2), self.nsnk_phs)
        self.tb.connect((self.fll, 3), self.nsnk_err)
        self.tb.run()

        N = 700
        dst_data = self.vsnk_frq.data()[N:]

        expected_result = len(dst_data) * [-0.20, ]
        self.assertFloatTuplesAlmostEqual(expected_result, dst_data, 4)

    def test02_noisy(self):
        sps = 4
        rolloff = 0.35
        bw = 2 * math.pi / 100.0
        ntaps = 45
        noise_std = 1 / 16
        length = 2 * 10**5

        # Create pulse shape filter
        rrc_taps = filter.firdes.root_raised_cosine(
            sps, sps, 1.0, rolloff, ntaps)

        # The frequency offset to correct
        foffset = 0.2 / (2.0 * math.pi)

        # Create a set of 1's and -1's, pulse shape and interpolate to sps
        random.seed(42)  # Fix seed
        data = [2.0 * random.randint(0, 2) - 1.0 for i in range(length)]
        noise = numpy.random.standard_normal(size=len(data)) + 1j * numpy.random.standard_normal(size=len(data))
        noise *= noise_std
        self.src = blocks.vector_source_c(numpy.array(data) + noise, False)
        self.rrc = filter.interp_fir_filter_ccf(sps, rrc_taps)

        # Mix symbols with a complex sinusoid to spin them
        self.nco = analog.sig_source_c(1, analog.GR_SIN_WAVE, foffset, 1)
        self.mix = blocks.multiply_cc()

        # FLL will despin the symbols to an arbitrary phase
        self.fll = digital.fll_band_edge_cc(sps, rolloff, ntaps, bw)

        # Create sinks for all outputs of the FLL
        # we will only care about the freq and error outputs
        self.vsnk_frq = blocks.vector_sink_f()
        self.nsnk_fll = blocks.null_sink(gr.sizeof_gr_complex)
        self.nsnk_phs = blocks.null_sink(gr.sizeof_float)
        self.nsnk_err = blocks.null_sink(gr.sizeof_float)

        # Connect the blocks
        self.tb.connect(self.nco, (self.mix, 1))
        self.tb.connect(self.src, self.rrc, (self.mix, 0))
        self.tb.connect(self.mix, self.fll, self.nsnk_fll)
        self.tb.connect((self.fll, 1), self.vsnk_frq)
        self.tb.connect((self.fll, 2), self.nsnk_phs)
        self.tb.connect((self.fll, 3), self.nsnk_err)
        self.tb.run()

        N = int(noise_std * 10000)
        self.assertLess(N, len(self.vsnk_frq.data()))
        dst_data = self.vsnk_frq.data()[N:]

        expected_result = len(dst_data) * [-0.20, ]
        self.assertFloatTuplesAlmostEqual(expected_result, dst_data, math.ceil(-math.log10(noise_std)))


if __name__ == '__main__':
    gr_unittest.run(test_fll_band_edge_cc)



#!/usr/bin/env python
#
# Copyright 2008,2010,2012,2013 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#


from gnuradio import gr, gr_unittest, filter, blocks

import cmath
import math


def fir_filter(x, taps, decim=1):
    y = []
    x2 = (len(taps) - 1) * [0, ] + x
    for i in range(0, len(x), decim):
        yi = 0
        for j in range(len(taps)):
            yi += taps[len(taps) - 1 - j] * x2[i + j]
        y.append(yi)
    return y


def sig_source_s(samp_rate, freq, amp, N):
    t = [float(x) / samp_rate for x in range(N)]
    y = [int(100 * math.sin(2. * math.pi * freq * x)) for x in t]
    return y


def sig_source_c(samp_rate, freq, amp, N):
    t = [float(x) / samp_rate for x in range(N)]
    y = [math.cos(2. * math.pi * freq * x) +
         1j * math.sin(2. * math.pi * freq * x) for x in t]
    return y


def mix(lo, data, phase=0.0):
    cphase = cmath.exp(1j * phase)
    y = [lo_i * data_i * cphase for lo_i, data_i in zip(lo, data)]
    return y


class test_freq_xlating_filter(gr_unittest.TestCase):

    def setUp(self):
        self.tb = gr.top_block()

    def tearDown(self):
        self.tb = None

    def generate_ccf_source(self):
        self.fs = fs = 1
        self.fc = fc = 0.3
        self.bw = bw = 0.1
        self.taps = filter.firdes.low_pass(1, fs, bw, bw / 4)
        times = list(range(100))
        self.src_data = [
            cmath.exp(-2j * cmath.pi * fc / fs * (t / 100.0)) for t in times]

    def generate_ccc_source(self):
        self.fs = fs = 1
        self.fc = fc = 0.3
        self.bw = bw = 0.1
        self.taps = filter.firdes.complex_band_pass(
            1, fs, -bw / 2, bw / 2, bw / 4)
        times = list(range(100))
        self.src_data = [
            cmath.exp(-2j * cmath.pi * fc / fs * (t / 100.0)) for t in times]

    def generate_fcf_source(self):
        self.fs = fs = 1
        self.fc = fc = 0.3
        self.bw = bw = 0.1
        self.taps = filter.firdes.low_pass(1, fs, bw, bw / 4)
        times = list(range(100))
        self.src_data = [
            math.sin(2 * cmath.pi * fc / fs * (t / 100.0)) for t in times]

    def generate_fcc_source(self):
        self.fs = fs = 1
        self.fc = fc = 0.3
        self.bw = bw = 0.1
        self.taps = filter.firdes.complex_band_pass(
            1, fs, -bw / 2, bw / 2, bw / 4)
        times = list(range(100))
        self.src_data = [
            math.sin(2 * cmath.pi * fc / fs * (t / 100.0)) for t in times]

    def generate_scf_source(self):
        self.fs = fs = 1
        self.fc = fc = 0.3
        self.bw = bw = 0.12
        self.taps = filter.firdes.low_pass(1, fs, bw, bw / 4)
        times = list(range(100))
        self.src_data = [
            int(100 * math.sin(2 * cmath.pi * fc / fs * (t / 100.0))) for t in times]

    def generate_scc_source(self):
        self.fs = fs = 1
        self.fc = fc = 0.3
        self.bw = bw = 0.12
        self.taps = filter.firdes.complex_band_pass(
            1, fs, -bw / 2, bw / 2, bw / 4)
        times = list(range(100))
        self.src_data = [
            int(100 * math.sin(2 * cmath.pi * fc / fs * (t / 100.0))) for t in times]

    def test_fir_filter_ccf_001(self):
        self.generate_ccf_source()

        decim = 1
        lo = sig_source_c(self.fs, -self.fc, 1, len(self.src_data))
        phase = -cmath.pi * self.fc / self.fs * (len(self.taps) - 1)
        despun = mix(lo, self.src_data, phase=phase)
        expected_data = fir_filter(despun, self.taps, decim)

        src = blocks.vector_source_c(self.src_data)
        op = filter.freq_xlating_fir_filter_ccf(
            decim, self.taps, self.fc, self.fs)
        dst = blocks.vector_sink_c()
        self.tb.connect(src, op, dst)
        self.tb.run()
        result_data = dst.data()

        self.assertComplexTuplesAlmostEqual(expected_data, result_data, 5)

    def test_fir_filter_ccf_002(self):
        self.generate_ccf_source()

        decim = 4
        lo = sig_source_c(self.fs, -self.fc, 1, len(self.src_data))
        phase = -cmath.pi * self.fc / self.fs * (len(self.taps) - 1)
        despun = mix(lo, self.src_data, phase=phase)
        expected_data = fir_filter(despun, self.taps, decim)

        src = blocks.vector_source_c(self.src_data)
        op = filter.freq_xlating_fir_filter_ccf(
            decim, self.taps, self.fc, self.fs)
        dst = blocks.vector_sink_c()
        self.tb.connect(src, op, dst)
        self.tb.run()
        result_data = dst.data()
        self.assertComplexTuplesAlmostEqual(expected_data, result_data, 5)

    def test_fir_filter_ccc_001(self):
        self.generate_ccc_source()

        decim = 1
        lo = sig_source_c(self.fs, -self.fc, 1, len(self.src_data))
        phase = -cmath.pi * self.fc / self.fs * (len(self.taps) - 1)
        despun = mix(lo, self.src_data, phase=phase)
        expected_data = fir_filter(despun, self.taps, decim)

        src = blocks.vector_source_c(self.src_data)
        op = filter.freq_xlating_fir_filter_ccc(
            decim, self.taps, self.fc, self.fs)
        dst = blocks.vector_sink_c()
        self.tb.connect(src, op, dst)
        self.tb.run()
        result_data = dst.data()
        self.assertComplexTuplesAlmostEqual(expected_data, result_data, 5)

    def test_fir_filter_ccc_002(self):
        self.generate_ccc_source()

        decim = 4
        lo = sig_source_c(self.fs, -self.fc, 1, len(self.src_data))
        phase = -cmath.pi * self.fc / self.fs * (len(self.taps) - 1)
        despun = mix(lo, self.src_data, phase=phase)
        expected_data = fir_filter(despun, self.taps, decim)

        src = blocks.vector_source_c(self.src_data)
        op = filter.freq_xlating_fir_filter_ccc(
            decim, self.taps, self.fc, self.fs)
        dst = blocks.vector_sink_c()
        self.tb.connect(src, op, dst)
        self.tb.run()
        result_data = dst.data()
        self.assertComplexTuplesAlmostEqual(expected_data, result_data, 5)

    def test_fir_filter_fcf_001(self):
        self.generate_fcf_source()

        decim = 1
        lo = sig_source_c(self.fs, -self.fc, 1, len(self.src_data))
        phase = -cmath.pi * self.fc / self.fs * (len(self.taps) - 1)
        despun = mix(lo, self.src_data, phase=phase)
        expected_data = fir_filter(despun, self.taps, decim)

        src = blocks.vector_source_f(self.src_data)
        op = filter.freq_xlating_fir_filter_fcf(
            decim, self.taps, self.fc, self.fs)
        dst = blocks.vector_sink_c()
        self.tb.connect(src, op, dst)
        self.tb.run()
        result_data = dst.data()
        self.assertComplexTuplesAlmostEqual(expected_data, result_data, 5)

    def test_fir_filter_fcf_002(self):
        self.generate_fcf_source()

        decim = 4
        lo = sig_source_c(self.fs, -self.fc, 1, len(self.src_data))
        phase = -cmath.pi * self.fc / self.fs * (len(self.taps) - 1)
        despun = mix(lo, self.src_data, phase=phase)
        expected_data = fir_filter(despun, self.taps, decim)

        src = blocks.vector_source_f(self.src_data)
        op = filter.freq_xlating_fir_filter_fcf(
            decim, self.taps, self.fc, self.fs)
        dst = blocks.vector_sink_c()
        self.tb.connect(src, op, dst)
        self.tb.run()
        result_data = dst.data()
        self.assertComplexTuplesAlmostEqual(expected_data, result_data, 5)

    def test_fir_filter_fcc_001(self):
        self.generate_fcc_source()

        decim = 1
        lo = sig_source_c(self.fs, -self.fc, 1, len(self.src_data))
        phase = -cmath.pi * self.fc / self.fs * (len(self.taps) - 1)
        despun = mix(lo, self.src_data, phase=phase)
        expected_data = fir_filter(despun, self.taps, decim)

        src = blocks.vector_source_f(self.src_data)
        op = filter.freq_xlating_fir_filter_fcc(
            decim, self.taps, self.fc, self.fs)
        dst = blocks.vector_sink_c()
        self.tb.connect(src, op, dst)
        self.tb.run()
        result_data = dst.data()
        self.assertComplexTuplesAlmostEqual(expected_data, result_data, 5)

    def test_fir_filter_fcc_002(self):
        self.generate_fcc_source()

        decim = 4
        lo = sig_source_c(self.fs, -self.fc, 1, len(self.src_data))
        phase = -cmath.pi * self.fc / self.fs * (len(self.taps) - 1)
        despun = mix(lo, self.src_data, phase=phase)
        expected_data = fir_filter(despun, self.taps, decim)

        src = blocks.vector_source_f(self.src_data)
        op = filter.freq_xlating_fir_filter_fcc(
            decim, self.taps, self.fc, self.fs)
        dst = blocks.vector_sink_c()
        self.tb.connect(src, op, dst)
        self.tb.run()
        result_data = dst.data()
        self.assertComplexTuplesAlmostEqual(expected_data, result_data, 5)

    def test_fir_filter_scf_001(self):
        self.generate_scf_source()

        decim = 1
        lo = sig_source_c(self.fs, -self.fc, 1, len(self.src_data))
        despun = mix(lo, self.src_data)
        expected_data = fir_filter(despun, self.taps, decim)

        src = blocks.vector_source_s(self.src_data)
        op = filter.freq_xlating_fir_filter_scf(
            decim, self.taps, self.fc, self.fs)
        dst = blocks.vector_sink_c()
        self.tb.connect(src, op, dst)
        self.tb.run()
        result_data = dst.data()
        self.assertComplexTuplesAlmostEqual(expected_data, result_data, 4)

    def test_fir_filter_scf_002(self):
        self.generate_scf_source()

        decim = 4
        lo = sig_source_c(self.fs, -self.fc, 1, len(self.src_data))
        despun = mix(lo, self.src_data)
        expected_data = fir_filter(despun, self.taps, decim)

        src = blocks.vector_source_s(self.src_data)
        op = filter.freq_xlating_fir_filter_scf(
            decim, self.taps, self.fc, self.fs)
        dst = blocks.vector_sink_c()
        self.tb.connect(src, op, dst)
        self.tb.run()
        result_data = dst.data()
        self.assertComplexTuplesAlmostEqual(expected_data, result_data, 4)

    def test_fir_filter_scc_001(self):
        self.generate_scc_source()

        decim = 1
        lo = sig_source_c(self.fs, -self.fc, 1, len(self.src_data))
        despun = mix(lo, self.src_data)
        expected_data = fir_filter(despun, self.taps, decim)

        src = blocks.vector_source_s(self.src_data)
        op = filter.freq_xlating_fir_filter_scc(
            decim, self.taps, self.fc, self.fs)
        dst = blocks.vector_sink_c()
        self.tb.connect(src, op, dst)
        self.tb.run()
        result_data = dst.data()
        self.assertComplexTuplesAlmostEqual(expected_data, result_data, 4)

    def test_fir_filter_scc_002(self):
        self.generate_scc_source()

        decim = 4
        lo = sig_source_c(self.fs, -self.fc, 1, len(self.src_data))
        despun = mix(lo, self.src_data)
        expected_data = fir_filter(despun, self.taps, decim)

        src = blocks.vector_source_s(self.src_data)
        op = filter.freq_xlating_fir_filter_scc(
            decim, self.taps, self.fc, self.fs)
        dst = blocks.vector_sink_c()
        self.tb.connect(src, op, dst)
        self.tb.run()
        result_data = dst.data()
        self.assertComplexTuplesAlmostEqual(expected_data, result_data, 4)


if __name__ == '__main__':
    gr_unittest.run(test_freq_xlating_filter)



#!/usr/bin/env python
#
# Copyright 2010,2013 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#


import numpy

from gnuradio import gr, gr_unittest, digital, analog, blocks


class test_cpm(gr_unittest.TestCase):

    def setUp(self):
        self.tb = gr.top_block()

    def tearDown(self):
        self.tb = None

    def do_check_phase_shift(self, type, name):
        sps = 2
        L = 1
        in_bits = (1,) * 20
        src = blocks.vector_source_b(in_bits, False)
        cpm = digital.cpmmod_bc(type, 0.5, sps, L)
        arg = blocks.complex_to_arg()
        sink = blocks.vector_sink_f()

        self.tb.connect(src, cpm, arg, sink)
        self.tb.run()

        symbol_phases = numpy.array(sink.data()[sps * L - 1::sps])
        phase_diff = numpy.mod(numpy.subtract(symbol_phases[1:], symbol_phases[:-1]),
                               (2 * numpy.pi,) * (len(symbol_phases) - 1))
        self.assertFloatTuplesAlmostEqual(
            tuple(phase_diff),
            (0.5 *
             numpy.pi,
             ) *
            len(phase_diff),
            5,
            msg="Phase shift was not correct for CPM method " +
            name)

    def test_001_lrec(self):
        self.do_check_phase_shift(analog.cpm.LRC, 'LREC')

    def test_001_lrc(self):
        self.do_check_phase_shift(analog.cpm.LRC, 'LRC')

    def test_001_lsrc(self):
        self.do_check_phase_shift(analog.cpm.LSRC, 'LSRC')

    def test_001_ltfm(self):
        self.do_check_phase_shift(analog.cpm.TFM, 'TFM')

    def test_001_lgmsk(self):
        sps = 2
        L = 5
        bt = 0.3
        in_bits = (1,) * 20
        src = blocks.vector_source_b(in_bits, False)
        gmsk = digital.gmskmod_bc(sps, L, bt)
        arg = blocks.complex_to_arg()
        sink = blocks.vector_sink_f()

        self.tb.connect(src, gmsk, arg, sink)
        self.tb.run()

        symbol_phases = numpy.array(sink.data()[sps * L - 1::sps])
        phase_diff = numpy.mod(numpy.subtract(symbol_phases[1:], symbol_phases[:-1]),
                               (2 * numpy.pi,) * (len(symbol_phases) - 1))
        self.assertFloatTuplesAlmostEqual(
            tuple(phase_diff),
            (0.5 * numpy.pi,
             ) * len(phase_diff),
            5,
            msg="Phase shift was not correct for GMSK")

    def test_phase_response(self):
        phase_response = analog.cpm.phase_response(analog.cpm.LREC, 2, 4)
        self.assertAlmostEqual(numpy.sum(phase_response), 1)


if __name__ == '__main__':
    gr_unittest.run(test_cpm)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#


import pmt
from gnuradio import gr, gr_unittest
from gnuradio import blocks


class qa_tsb_vector_sink (gr_unittest.TestCase):

    def setUp(self):
        self.tb = gr.top_block()
        self.tsb_key = "tsb"

    def tearDown(self):
        self.tb = None

    def test_001_t(self):
        packet_len = 4
        data = list(range(2 * packet_len))
        tag = gr.tag_t()
        tag.key = pmt.intern("foo")
        tag.offset = 5
        tag.value = pmt.intern("bar")
        src = blocks.vector_source_f(data, tags=(tag,))
        sink = blocks.tsb_vector_sink_f(tsb_key=self.tsb_key)
        self.tb.connect(
            src,
            blocks.stream_to_tagged_stream(
                gr.sizeof_float,
                1,
                packet_len,
                self.tsb_key),
            sink)
        self.tb.run()
        self.assertEqual([data[0:packet_len], data[packet_len:]], sink.data())
        self.assertEqual(len(sink.tags()), 1)
        self.assertEqual(sink.tags()[0].offset, tag.offset)


if __name__ == '__main__':
    gr_unittest.run(qa_tsb_vector_sink)



#!/usr/bin/env python
#
# Copyright 2021 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from gnuradio import gr, gr_unittest, blocks, network
import time


class qa_udp_source (gr_unittest.TestCase):
    def setUp(self):
        self.tb = gr.top_block()

    def tearDown(self):
        self.tb = None

    def test_restart(self):
        udp_source = network.udp_source(gr.sizeof_gr_complex, 1, 1234, 0, 1472,
                                        False, False, False)
        null_sink = blocks.null_sink(gr.sizeof_gr_complex)
        self.tb.connect(udp_source, null_sink)
        self.tb.start()
        time.sleep(0.1)
        self.tb.stop()
        time.sleep(0.1)
        self.tb.start()
        time.sleep(0.1)
        self.tb.stop()


if __name__ == '__main__':
    gr_unittest.run(qa_udp_source)



#!/usr/bin/env python
#
# Copyright 2004,2007,2010-2013 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#


from gnuradio import gr, gr_unittest, digital, blocks


class test_constellation_decoder(gr_unittest.TestCase):

    def setUp(self):
        self.tb = gr.top_block()

    def tearDown(self):
        self.tb = None

    def test_constellation_decoder_cb_bpsk(self):
        cnst = digital.constellation_bpsk()
        src_data = (0.5 + 0.5j, 0.1 - 1.2j, -0.8 - 0.1j, -0.45 + 0.8j,
                    0.8 + 1.0j, -0.5 + 0.1j, 0.1 - 1.2j)
        expected_result = (1, 1, 0, 0,
                           1, 0, 1)
        src = blocks.vector_source_c(src_data)
        op = digital.constellation_decoder_cb(cnst.base())
        dst = blocks.vector_sink_b()

        self.tb.connect(src, op)
        self.tb.connect(op, dst)
        self.tb.run()               # run the graph and wait for it to finish

        actual_result = dst.data()  # fetch the contents of the sink
        # print "actual result", actual_result
        # print "expected result", expected_result
        self.assertFloatTuplesAlmostEqual(expected_result, actual_result)

    def _test_constellation_decoder_cb_qpsk(self):
        cnst = digital.constellation_qpsk()
        src_data = (0.5 + 0.5j, 0.1 - 1.2j, -0.8 - 0.1j, -0.45 + 0.8j,
                    0.8 + 1.0j, -0.5 + 0.1j, 0.1 - 1.2j)
        expected_result = (3, 1, 0, 2,
                           3, 2, 1)
        src = blocks.vector_source_c(src_data)
        op = digital.constellation_decoder_cb(cnst.base())
        dst = blocks.vector_sink_b()

        self.tb.connect(src, op)
        self.tb.connect(op, dst)
        self.tb.run()               # run the graph and wait for it to finish

        actual_result = dst.data()  # fetch the contents of the sink
        # print "actual result", actual_result
        # print "expected result", expected_result
        self.assertFloatTuplesAlmostEqual(expected_result, actual_result)

    def test_constellation_decoder_cb_bpsk_to_qpsk(self):
        cnst = digital.constellation_bpsk()
        src_data = (0.5 + 0.5j, 0.1 - 1.2j, -0.8 - 0.1j, -0.45 + 0.8j,
                    0.8 + 1.0j, -0.5 + 0.1j, 0.1 - 1.2j)
        src = blocks.vector_source_c(src_data)
        op = digital.constellation_decoder_cb(cnst.base())
        dst = blocks.vector_sink_b()

        self.tb.connect(src, op)
        self.tb.connect(op, dst)
        self.tb.run()               # run the graph and wait for it to finish

        cnst = digital.constellation_qpsk()
        src_data = (0.5 + 0.5j, 0.1 - 1.2j, -0.8 - 0.1j, -0.45 + 0.8j,
                    0.8 + 1.0j, -0.5 + 0.1j, 0.1 - 1.2j)
        expected_result = (1, 1, 0, 0,  # BPSK section
                           1, 0, 1,
                           3, 1, 0, 2,  # QPSK section
                           3, 2, 1)
        src.set_data(src_data)
        op.set_constellation(cnst.base())

        self.tb.run()               # run the graph and wait for it to finish

        actual_result = dst.data()  # fetch the contents of the sink
        # print("actual result", actual_result)
        # print("expected result", expected_result)
        self.assertFloatTuplesAlmostEqual(expected_result, actual_result)


if __name__ == '__main__':
    gr_unittest.run(test_constellation_decoder)



#!/usr/bin/env python
#
# Copyright 2013 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#


from gnuradio import gr, gr_unittest, blocks
import math


class test_null_sink_source(gr_unittest.TestCase):

    def setUp(self):
        self.tb = gr.top_block()

    def tearDown(self):
        self.tb = None

    def test_001(self):
        # Just running some data through null source/sink
        src = blocks.null_source(gr.sizeof_float)
        hed = blocks.head(gr.sizeof_float, 100)
        dst = blocks.null_sink(gr.sizeof_float)

        self.tb.connect(src, hed, dst)
        self.tb.run()


if __name__ == '__main__':
    gr_unittest.run(test_null_sink_source)



#!/usr/bin/env python
#
# Copyright 2004,2007,2010,2012,2013 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#


from gnuradio import gr, gr_unittest, blocks


class test_add_mult_div_sub(gr_unittest.TestCase):

    def setUp(self):
        self.tb = gr.top_block()

    def tearDown(self):
        self.tb = None

    def help_ii(self, src_data, exp_data, op):
        for s in zip(list(range(len(src_data))), src_data):
            src = blocks.vector_source_i(s[1])
            self.tb.connect(src, (op, s[0]))
        dst = blocks.vector_sink_i()
        self.tb.connect(op, dst)
        self.tb.run()
        result_data = dst.data()
        self.assertEqual(exp_data, result_data)

    def help_ss(self, src_data, exp_data, op):
        for s in zip(list(range(len(src_data))), src_data):
            src = blocks.vector_source_s(s[1])
            self.tb.connect(src, (op, s[0]))
        dst = blocks.vector_sink_s()
        self.tb.connect(op, dst)
        self.tb.run()
        result_data = dst.data()
        self.assertEqual(exp_data, result_data)

    def help_ff(self, src_data, exp_data, op):
        for s in zip(list(range(len(src_data))), src_data):
            src = blocks.vector_source_f(s[1])
            self.tb.connect(src, (op, s[0]))
        dst = blocks.vector_sink_f()
        self.tb.connect(op, dst)
        self.tb.run()
        result_data = dst.data()
        self.assertEqual(exp_data, result_data)

    def help_cc(self, src_data, exp_data, op):
        for s in zip(list(range(len(src_data))), src_data):
            src = blocks.vector_source_c(s[1])
            self.tb.connect(src, (op, s[0]))
        dst = blocks.vector_sink_c()
        self.tb.connect(op, dst)
        self.tb.run()
        result_data = dst.data()
        self.assertEqual(exp_data, result_data)

    # add_XX

    def test_add_ss(self):
        src1_data = [1, 2, 3, 4, 5]
        src2_data = [8, -3, 4, 8, 2]
        expected_result = [9, -1, 7, 12, 7]
        op = blocks.add_ss()
        self.help_ss((src1_data, src2_data), expected_result, op)

    def test_add_ii(self):
        src1_data = [1, 2, 3, 4, 5]
        src2_data = [8, -3, 4, 8, 2]
        expected_result = [9, -1, 7, 12, 7]
        op = blocks.add_ii()
        self.help_ii((src1_data, src2_data), expected_result, op)

    def test_add_ff(self):
        src1_data = [1.0, 2.0, 3.0, 4.0, 5.0]
        src2_data = [8.0, -3.0, 4.0, 8.0, 2.0]
        expected_result = [9.0, -1.0, 7.0, 12.0, 7.0]
        op = blocks.add_ff()
        self.help_ff((src1_data, src2_data), expected_result, op)

    def test_add_cc(self):
        src1_data = [1 + 1j, 2 + 2j, 3 + 3j, 4 + 4j, 5 + 5j]
        src2_data = [8 + 8j, -3 - 3j, 4 + 4j, 8 + 8j, 2 + 2j]
        expected_result = [9 + 9j, -1 - 1j, 7 + 7j, 12 + 12j, 7 + 7j]
        op = blocks.add_cc()
        self.help_cc((src1_data, src2_data), expected_result, op)

    # add_const_XX

    def test_add_const_ss(self):
        src_data = [1, 2, 3, 4, 5]
        expected_result = [6, 7, 8, 9, 10]
        op = blocks.add_const_ss(5)
        self.help_ss((src_data,), expected_result, op)

    def test_add_const_ii(self):
        src_data = [1, 2, 3, 4, 5]
        expected_result = [6, 7, 8, 9, 10]
        op = blocks.add_const_ii(5)
        self.help_ii((src_data,), expected_result, op)

    def test_add_const_ff(self):
        src_data = [1, 2, 3, 4, 5]
        expected_result = [6, 7, 8, 9, 10]
        op = blocks.add_const_ff(5)
        self.help_ff((src_data,), expected_result, op)

    def test_add_const_cc(self):
        src_data = [1, 2, 3, 4, 5]
        expected_result = [1 + 5j, 2 + 5j, 3 + 5j, 4 + 5j, 5 + 5j]
        op = blocks.add_const_cc(5j)
        self.help_cc((src_data,), expected_result, op)

    # multiply_XX

    def test_multiply_ss(self):
        src1_data = [1, 2, 3, 4, 5]
        src2_data = [8, -3, 4, 8, 2]
        expected_result = [8, -6, 12, 32, 10]
        op = blocks.multiply_ss()
        self.help_ss((src1_data, src2_data),
                     expected_result, op)

    def test_multiply_ii(self):
        src1_data = [1, 2, 3, 4, 5]
        src2_data = [8, -3, 4, 8, 2]
        expected_result = [8, -6, 12, 32, 10]
        op = blocks.multiply_ii()
        self.help_ii((src1_data, src2_data),
                     expected_result, op)

    def test_multiply_ff(self):
        src1_data = [1, 2, 3, 4, 5]
        src2_data = [8, -3, 4, 8, 2]
        expected_result = [8, -6, 12, 32, 10]
        op = blocks.multiply_ff()
        self.help_ff((src1_data, src2_data),
                     expected_result, op)

    def test_multiply_cc(self):
        src1_data = [1 + 1j, 2 + 2j, 3 + 3j, 4 + 4j, 5 + 5j]
        src2_data = [8, -3, 4, 8, 2]
        expected_result = [8 + 8j, -6 - 6j, 12 + 12j, 32 + 32j, 10 + 10j]
        op = blocks.multiply_cc()
        self.help_cc((src1_data, src2_data),
                     expected_result, op)

    # multiply_const_XX

    def test_multiply_const_ss(self):
        src_data = [-1, 0, 1, 2, 3]
        expected_result = [-5, 0, 5, 10, 15]
        op = blocks.multiply_const_ss(5)
        self.help_ss((src_data,), expected_result, op)

    def test_multiply_const_ii(self):
        src_data = [-1, 0, 1, 2, 3]
        expected_result = [-5, 0, 5, 10, 15]
        op = blocks.multiply_const_ii(5)
        self.help_ii((src_data,), expected_result, op)

    def test_multiply_const_ff(self):
        src_data = [-1, 0, 1, 2, 3]
        expected_result = [-5, 0, 5, 10, 15]
        op = blocks.multiply_const_ff(5)
        self.help_ff((src_data,), expected_result, op)

    def test_multiply_const_cc(self):
        src_data = [-1 - 1j, 0 + 0j, 1 + 1j, 2 + 2j, 3 + 3j]
        expected_result = [-5 - 5j, 0 + 0j, 5 + 5j, 10 + 10j, 15 + 15j]
        op = blocks.multiply_const_cc(5)
        self.help_cc((src_data,), expected_result, op)

    def test_multiply_const_cc2(self):
        src_data = [-1 - 1j, 0 + 0j, 1 + 1j, 2 + 2j, 3 + 3j]
        expected_result = [-3 - 7j, 0 + 0j, 3 + 7j, 6 + 14j, 9 + 21j]
        op = blocks.multiply_const_cc(5 + 2j)
        self.help_cc((src_data,), expected_result, op)

    def test_sub_ii(self):
        src1_data = [1, 2, 3, 4, 5]
        src2_data = [8, -3, 4, 8, 2]
        expected_result = [-7, 5, -1, -4, 3]
        op = blocks.sub_ii()
        self.help_ii((src1_data, src2_data),
                     expected_result, op)

    def test_sub_ii1(self):
        src1_data = [1, 2, 3, 4, 5]
        expected_result = [1, 2, 3, 4, 5]

        src = blocks.vector_source_i(src1_data)
        op = blocks.sub_ii()
        dst = blocks.vector_sink_i()
        self.tb.connect(src, op, dst)
        self.tb.run()
        result_data = dst.data()
        self.assertEqual(expected_result, result_data)

    def test_sub_ss(self):
        src1_data = [1, 2, 3, 4, 5]
        src2_data = [8, -3, 4, 8, 2]
        expected_result = [-7, 5, -1, -4, 3]
        op = blocks.sub_ss()
        self.help_ss((src1_data, src2_data),
                     expected_result, op)

    def test_sub_ss1(self):
        src1_data = [1, 2, 3, 4, 5]
        expected_result = [1, 2, 3, 4, 5]

        src = blocks.vector_source_s(src1_data)
        op = blocks.sub_ss()
        dst = blocks.vector_sink_s()
        self.tb.connect(src, op, dst)
        self.tb.run()
        result_data = dst.data()
        self.assertEqual(expected_result, result_data)

    def test_sub_ff(self):
        src1_data = [1, 2, 3, 4, 5]
        src2_data = [8, -3, 4, 8, 2]
        expected_result = [-7, 5, -1, -4, 3]
        op = blocks.sub_ff()
        self.help_ff((src1_data, src2_data),
                     expected_result, op)

    def test_sub_ff1(self):
        src1_data = [1, 2, 3, 4, 5]
        expected_result = [1, 2, 3, 4, 5]

        src = blocks.vector_source_f(src1_data)
        op = blocks.sub_ff()
        dst = blocks.vector_sink_f()
        self.tb.connect(src, op, dst)
        self.tb.run()
        result_data = dst.data()
        self.assertEqual(expected_result, result_data)

    def test_sub_cc(self):
        src1_data = [1, 2, 3, 4, 5]
        src2_data = [8, -3, 4, 8, 2]
        expected_result = [-7, 5, -1, -4, 3]
        op = blocks.sub_cc()
        self.help_cc((src1_data, src2_data),
                     expected_result, op)

    def test_sub_cc1(self):
        src1_data = [1, 2, 3, 4, 5]
        expected_result = [1, 2, 3, 4, 5]

        src = blocks.vector_source_c(src1_data)
        op = blocks.sub_cc()
        dst = blocks.vector_sink_c()
        self.tb.connect(src, op, dst)
        self.tb.run()
        result_data = dst.data()
        self.assertEqual(expected_result, result_data)

    def test_div_ff(self):
        src1_data = [5, 9, -15, 1024]
        src2_data = [10, 3, -5, 64]
        expected_result = [0.5, 3, 3, 16]
        op = blocks.divide_ff()
        self.help_ff((src1_data, src2_data), expected_result, op)


if __name__ == '__main__':
    gr_unittest.run(test_add_mult_div_sub)



"""
Copyright 2023 Ettus Research, A National Instruments Brand.

SPDX-License-Identifier: GPL-3.0-or-later
"""

import sys

MAX_NUM_CHANNELS = 4

MAIN_TMPL = """\
id: uhd_rfnoc_duc
label: RFNoC Digital Upconverter Block
category: '[Core]/UHD/RFNoC/Blocks'

parameters:
- id: num_chans
  label: Channel count
  dtype: int
  default: 1
  hide: ${'$'}{ 'part' }
- id: block_args
  label: Block Args
  dtype: string
  default: ""
  hide: ${'$'}{ 'part' if not block_args else 'none'}
- id: device_select
  label: Device Select
  dtype: int
  default: -1
  hide: ${'$'}{ 'part' if device_select == -1 else 'none'}
- id: instance_index
  label: Instance Select
  dtype: int
  default: -1
  hide: ${'$'}{ 'part' if instance_index == -1 else 'none'}
${coeffs_params}

inputs:
- domain: rfnoc
  dtype: 'sc16'
  vlen: 1
  multiplicity: ${'$'}{num_chans}

outputs:
- domain: rfnoc
  dtype: 'sc16'
  vlen: 1
  multiplicity: ${'$'}{num_chans}

templates:
  imports: |-
    from gnuradio import uhd
  make: |-
    uhd.rfnoc_duc(
        self.rfnoc_graph,
        uhd.device_addr(${'$'}{block_args}),
        ${'$'}{device_select},
        ${'$'}{instance_index})
${init_params}  callbacks:
${callback_params}

file_format: 1
"""

COEFFS_PARAM = """- id: freq${n}
  label: 'Ch${n}: Frequency Shift (Hz)'
  dtype: real
  default: 0
  hide: ${'$'}{ 'none' if num_chans > ${n} else 'all' }
- id: input_rate${n}
  label: 'Ch${n}: Input Rate (Hz)'
  dtype: real
  default: 0
  hide: ${'$'}{ 'none' if num_chans > ${n} else 'all' }
"""

INIT_PARAM = """    ${'%'} if context.get('num_chans')() > ${n}:
    self.${'$'}{id}.set_freq(${'$'}{freq${n}}, ${n})
    self.${'$'}{id}.set_input_rate(${'$'}{input_rate${n}}, ${n})
    ${'%'} endif
"""

CALLBACKS_PARAM = """  - |
    ${'%'} if context.get('num_chans')() > ${n}:
    set_freq(${'$'}{freq${n}}, ${n})
    ${'%'} endif
  - |
    ${'%'} if context.get('num_chans')() > ${n}:
    set_input_rate(${'$'}{input_rate${n}}, ${n})
    ${'%'} endif
"""


def parse_tmpl(_tmpl, **kwargs):
    """ Render _tmpl using the kwargs """
    from mako.template import Template

    block_template = Template(_tmpl)
    return str(block_template.render(**kwargs))


if __name__ == '__main__':
    file = sys.argv[1]
    coeffs_params = ''.join([
        parse_tmpl(COEFFS_PARAM, n=n)
        for n in range(MAX_NUM_CHANNELS)
    ])
    init_params = ''.join([
        parse_tmpl(INIT_PARAM, n=n)
        for n in range(MAX_NUM_CHANNELS)
    ])
    callback_params = ''.join([
        parse_tmpl(CALLBACKS_PARAM, n=n)
        for n in range(MAX_NUM_CHANNELS)
    ])
    open(file, 'w').write(
        parse_tmpl(
            MAIN_TMPL,
            max_num_chans=MAX_NUM_CHANNELS,
            coeffs_params=coeffs_params,
            init_params=init_params,
            callback_params=callback_params
        )
    )



#!/usr/bin/env python
#
# Copyright 2006,2007,2010,2012,2013 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#


import random

from gnuradio import gr, gr_unittest, digital, blocks


def make_random_int_list(L, min, max):
    result = []
    for x in range(L):
        result.append(random.randint(min, max))
    return list(result)


class test_diff_encoder(gr_unittest.TestCase):

    def setUp(self):
        random.seed(0)
        self.tb = gr.top_block()

    def tearDown(self):
        self.tb = None

    def test_diff_encdec_000(self):
        for modulus in (2, 4, 8):
            with self.subTest(modulus=modulus):
                src_data = make_random_int_list(40000, 0, modulus - 1)
                expected_result = src_data
                src = blocks.vector_source_b(src_data)
                enc = digital.diff_encoder_bb(modulus)
                dec = digital.diff_decoder_bb(modulus)
                dst = blocks.vector_sink_b()
                self.tb.connect(src, enc, dec, dst)
                self.tb.run()  # run the graph and wait for it to finish
                actual_result = dst.data()  # fetch the contents of the sink
                self.assertEqual(expected_result, actual_result)


if __name__ == '__main__':
    gr_unittest.run(test_diff_encoder)



#!/usr/bin/env python
#
# Copyright 2006,2010,2013 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#


import random

from gnuradio import gr, gr_unittest, blocks
import pmt


class test_pack(gr_unittest.TestCase):

    def setUp(self):
        random.seed(0)
        self.tb = gr.top_block()

    def tearDown(self):
        self.tb = None

    def test_001(self):
        src_data = [1, 0, 1, 1, 0, 1, 1, 0]
        expected_results = [1, 0, 1, 1, 0, 1, 1, 0]
        src = blocks.vector_source_b(src_data, False)
        op = blocks.pack_k_bits_bb(1)
        dst = blocks.vector_sink_b()
        self.tb.connect(src, op, dst)
        self.tb.run()
        self.assertEqual(expected_results, dst.data())

    def test_002(self):
        src_data = [1, 0, 1, 1, 0, 0, 0, 1]
        expected_results = [2, 3, 0, 1]
        src = blocks.vector_source_b(src_data, False)
        op = blocks.pack_k_bits_bb(2)
        dst = blocks.vector_sink_b()
        self.tb.connect(src, op, dst)
        self.tb.run()
        self.assertEqual(expected_results, dst.data())

    def test_003(self):
        src_data = expected_results = [random.randint(0, 3) for x in range(10)]
        src = blocks.vector_source_b(src_data)
        pack = blocks.pack_k_bits_bb(2)
        unpack = blocks.unpack_k_bits_bb(2)
        snk = blocks.vector_sink_b()
        self.tb.connect(src, unpack, pack, snk)
        self.tb.run()
        self.assertEqual(list(expected_results), list(snk.data()))

    def test_004(self):
        # Test tags propagation

        # Tags on the incoming bits
        src_data = [1, 0, 1, 1, 0, 0, 0, 1]
        #src_tag_offsets = [1, 2, 3, 5, 6]
        src_tag_offsets = [1, 2, 3, 5, 6, 7]

        # Ground Truth
        expected_data = [2, 3, 0, 1]
        expected_tag_offsets = [0, 1, 1, 2, 3, 3]

        test_tags = list()
        tag_indexs = range(len(src_tag_offsets))
        for src_tag in tag_indexs:
            test_tags.append(
                gr.tag_utils.python_to_tag((src_tag_offsets[src_tag],
                                            pmt.intern('tag_byte'),
                                            pmt.from_long(src_tag),
                                            None
                                            ))
            )

        src = blocks.vector_source_b(src_data, False, 1, test_tags)
        op = blocks.pack_k_bits_bb(2)
        dst = blocks.vector_sink_b()
        self.tb.connect(src, op, dst)
        self.tb.run()

        # Check the data
        self.assertEqual(expected_data, dst.data())

        # Check the tag values
        self.assertEqual(list(tag_indexs), [
                         pmt.to_python(x.value) for x in dst.tags()])

        # Check the tag offsets
        self.assertEqual(expected_tag_offsets, [x.offset for x in dst.tags()])


if __name__ == '__main__':
    gr_unittest.run(test_pack)



#!/usr/bin/env python
#
# Copyright 2007,2010,2013 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#


from gnuradio import gr, gr_unittest, blocks


class test_regenerate(gr_unittest.TestCase):

    def setUp(self):
        self.tb = gr.top_block()

    def tearDown(self):
        self.tb = None

    def test_regen1(self):
        tb = self.tb

        data = [0, 0, 0,
                1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
                1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0]

        expected_result = [
            0,
            0,
            0,
            1,
            0,
            0,
            0,
            0,
            1,
            0,
            0,
            0,
            0,
            1,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            1,
            0,
            0,
            0,
            0,
            1,
            0,
            0,
            0,
            0,
            1,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0]

        src = blocks.vector_source_b(data, False)
        regen = blocks.regenerate_bb(5, 2)
        dst = blocks.vector_sink_b()

        tb.connect(src, regen)
        tb.connect(regen, dst)
        tb.run()

        dst_data = dst.data()

        self.assertEqual(expected_result, dst_data)

    def test_regen2(self):
        tb = self.tb

        data = 200 * [0, ]
        data[9] = 1
        data[99] = 1

        expected_result = 200 * [0, ]
        expected_result[9] = 1
        expected_result[19] = 1
        expected_result[29] = 1
        expected_result[39] = 1

        expected_result[99] = 1
        expected_result[109] = 1
        expected_result[119] = 1
        expected_result[129] = 1

        src = blocks.vector_source_b(data, False)
        regen = blocks.regenerate_bb(10, 3)
        dst = blocks.vector_sink_b()

        tb.connect(src, regen)
        tb.connect(regen, dst)
        tb.run()

        dst_data = dst.data()

        self.assertEqual(expected_result, dst_data)


if __name__ == '__main__':
    gr_unittest.run(test_regenerate)



#
# Copyright 2019 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#
""" Module to get io_signature of the header block """


import re
import itertools
import logging
import string

from ..core import Constants

LOGGER = logging.getLogger(__name__)


def io_signature(impl_file):
    """
    function to generate the io_signature of the block
    : returns the io parameters
    """
    parsed_io = {
        "input": {
            "signature": None
        },
        "output": {
            "signature": None
        }
    }
    with open(impl_file, 'r') as impl:
        io_lines = []
        for line in impl:
            if Constants.IO_SIGNATURE in line:
                io_lines.append(line)
    if len(io_lines) > 2:
        io_lines = io_lines[0:2]
    _io_sig = []
    for line in io_lines:
        if Constants.IO_SIGNATURE in line:
            line = line.lstrip().rstrip().split(Constants.IO_SIGNATURE)
            _io_sig.append(line)
    _io_sig = list(itertools.chain.from_iterable(_io_sig))
    for index, _element in enumerate(_io_sig):
        _io_sig[index] = _element.lstrip().rstrip()
        if all(i in string.punctuation for i in _element):
            _io_sig.remove(_element)
    _io_sig = list(filter(None, _io_sig))
    io_func = []
    for _io in _io_sig:
        if Constants.MAKE in _io:
            io_func.append(_io.lstrip().rstrip(Constants.STRIP_SYMBOLS))
    for signature in Constants.SIGNATURE_LIST:
        if signature in io_func[0] and parsed_io['input']['signature'] is None:
            parsed_io['input']['signature'] = signature
            io_func[0] = io_func[0].lstrip(signature + ' (')
        if signature in io_func[1] and parsed_io['output']['signature'] is None:
            parsed_io['output']['signature'] = signature
            io_func[1] = io_func[1].lstrip(signature + ' (')
    io_elements = []
    for _io in io_func:
        _io = _io.split(',')
        io_elements.append(_io)
    io_elements = list(itertools.chain.from_iterable(io_elements))
    for index, _io in enumerate(io_elements):
        _io = _io.lstrip(' (').rstrip(' )')
        if Constants.OPEN_BRACKET in _io:
            _io = _io + Constants.CLOSE_BRACKET
        io_elements[index] = _io

    # Because of any possible combination of I/O signature and different number
    # of arguments, manual if else loop is required
    if parsed_io['input']['signature'] is Constants.MAKE:
        parsed_io['input']['min_streams'] = io_elements[0]
        parsed_io['input']['max_streams'] = io_elements[1]
        parsed_io['input']['sizeof_stream_item'] = io_elements[2]
        del io_elements[0:3]
    elif parsed_io['input']['signature'] is Constants.MAKE2:
        parsed_io['input']['min_streams'] = io_elements[0]
        parsed_io['input']['max_streams'] = io_elements[1]
        parsed_io['input']['sizeof_stream_item1'] = io_elements[2]
        parsed_io['input']['sizeof_stream_item2'] = io_elements[3]
        del io_elements[0:4]
    elif parsed_io['input']['signature'] is Constants.MAKE3:
        parsed_io['input']['min_streams'] = io_elements[0]
        parsed_io['input']['max_streams'] = io_elements[1]
        parsed_io['input']['sizeof_stream_item1'] = io_elements[2]
        parsed_io['input']['sizeof_stream_item2'] = io_elements[3]
        parsed_io['input']['sizeof_stream_item3'] = io_elements[4]
        del io_elements[0:5]
    elif parsed_io['input']['signature'] is Constants.MAKEV:
        parsed_io['input']['min_streams'] = io_elements[0]
        parsed_io['input']['max_streams'] = io_elements[1]
        parsed_io['input']['sizeof_stream_items'] = io_elements[2]
        del io_elements[0:3]

    if parsed_io['output']['signature'] is Constants.MAKE:
        parsed_io['output']['min_streams'] = io_elements[0]
        parsed_io['output']['max_streams'] = io_elements[1]
        parsed_io['output']['sizeof_stream_item'] = io_elements[2]
        del io_elements[0:3]
    elif parsed_io['output']['signature'] is Constants.MAKE2:
        parsed_io['output']['min_streams'] = io_elements[0]
        parsed_io['output']['max_streams'] = io_elements[1]
        parsed_io['output']['sizeof_stream_item1'] = io_elements[2]
        parsed_io['output']['sizeof_stream_item2'] = io_elements[3]
        del io_elements[0:4]
    elif parsed_io['output']['signature'] is Constants.MAKE3:
        parsed_io['output']['min_streams'] = io_elements[0]
        parsed_io['output']['max_streams'] = io_elements[1]
        parsed_io['output']['sizeof_stream_item1'] = io_elements[2]
        parsed_io['output']['sizeof_stream_item2'] = io_elements[3]
        parsed_io['output']['sizeof_stream_item3'] = io_elements[4]
        del io_elements[0:5]
    elif parsed_io['output']['signature'] is Constants.MAKEV:
        parsed_io['output']['min_streams'] = io_elements[0]
        parsed_io['output']['max_streams'] = io_elements[1]
        parsed_io['output']['sizeof_stream_items'] = io_elements[2]
        del io_elements[0:3]
    return parsed_io


def message_port(impl_file):
    """
    parses message ports from implementation file
    """
    parsed_message_port = {
        "input": [],
        "output": []
    }
    with open(impl_file, 'r') as impl:
        _input = []
        _output = []
        for line in impl:
            if Constants.MESSAGE_INPUT in line:
                _input.append(line)
            if Constants.MESSAGE_OUTPUT in line:
                _output.append(line)

    input_port = []
    output_port = []
    if _input:
        for port in _input:
            port = port.lstrip().rstrip().strip(Constants.MESSAGE_INPUT)
            pattern = port.find('\"')
            if pattern != -1:
                if re.findall(r'"([^"]*)"', port)[0]:
                    input_port.append(re.findall(r'"([^"]*)"', port)[0])
            else:
                input_port.append(port[port.find('(') + 1:port.rfind(')')])
                _temp_port = ''.join(map(str, input_port))
                input_port.clear()
                input_port.append(_temp_port)

    if _output:
        for port in _output:
            port = port.lstrip().rstrip().strip(Constants.MESSAGE_OUTPUT)
            pattern = port.find('\"')
            if pattern != -1:
                if re.findall(r'"([^"]*)"', port)[0]:
                    output_port.append(re.findall(r'"([^"]*)"', port)[0])
            else:
                output_port.append(port[port.find('(') + 1:port.rfind(')')])
                _temp_port = ''.join(map(str, output_port))
                output_port.clear()
                output_port.append(_temp_port)

    if input_port:
        for port in input_port:
            parsed_message_port['input'].append(port)

    if output_port:
        for port in output_port:
            parsed_message_port['output'].append(port)
    return parsed_message_port



#
# Copyright 2019 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#
""" Module to read and add special blocktool comments in the public header """


import warnings

from ..core import Constants


def strip_symbols(line):
    """
    helper function to strip symbols
    from blocktool comment syntax
    """
    return line.split(':')[-1].lstrip().rstrip()


def exist_comments(self):
    """
    function to check if blocktool special comments
    already exist in the public header
    """
    _comments = True
    _index = None
    lines = []
    with open(self.target_file, 'r') as header:
        lines = header.readlines()
        for line in lines:
            if Constants.BLOCKTOOL in line:
                _index = lines.index(line)
    return bool(_index)


def validate_message_port(self, message_ports, suppress_input, suppress_output):
    """
    function to solve conflicts if any in the
    *message_port* comments and the implementation information
    """
    if message_ports['input'] != self.parsed_data['message_port']['input']:
        if not suppress_input:
            warnings.warn(
                'Conflict in values input message port Id. Add ! at the start of the key-value line to mandatory use the comment value.')
            self.parsed_data['message_port']['input'] = message_ports['input']
    if message_ports['output'] != self.parsed_data['message_port']['output']:
        if not suppress_output:
            warnings.warn(
                'Conflict in values output message port Id. Add ! at the start of the key-value line to mandatory use the comment value.')
            self.parsed_data['message_port']['output'] = message_ports['output']


def read_comments(self):
    """
    function to read special blocktool comments
    in the public header
    """
    temp_parsed_data = {}
    if self.parsed_data['io_signature'] or self.parsed_data['message_port']:
        temp_parsed_data['io_signature'] = self.parsed_data['io_signature']
        temp_parsed_data['message_port'] = self.parsed_data['message_port']
    self.parsed_data['io_signature'] = {
        "input": {
            "signature": None
        },
        "output": {
            "signature": None
        }
    }
    self.parsed_data['message_port'] = {
        "input": [],
        "output": []
    }
    _suppress_input = False
    _suppress_output = False
    parsed_io = self.parsed_data['io_signature']
    message_port = self.parsed_data['message_port']
    special_comments = []
    _index = None
    lines = []
    with open(self.target_file, 'r') as header:
        lines = header.readlines()
        for line in lines:
            if Constants.BLOCKTOOL in line:
                _index = lines.index(line)

    if _index is not None:
        _index = _index + 1
        for num in range(_index, len(lines)):
            if Constants.END_BLOCKTOOL in lines[num]:
                break
            special_comments.append(lines[num])
        for comment in special_comments:
            if Constants.INPUT_SIG in comment:
                parsed_io['input']['signature'] = strip_symbols(comment)
            if Constants.INPUT_MAX in comment:
                parsed_io['input']['max_streams'] = strip_symbols(comment)
            if Constants.INPUT_MIN in comment:
                parsed_io['input']['min_streams'] = strip_symbols(comment)
            if parsed_io['input']['signature'] is Constants.MAKE and not None:
                if Constants.INPUT_MAKE_SIZE in comment:
                    parsed_io['input']['sizeof_stream_item'] = strip_symbols(
                        comment)
            elif parsed_io['input']['signature'] is Constants.MAKE2 and not None:
                if Constants.INPUT_MAKE_SIZE1 in comment:
                    parsed_io['input']['sizeof_stream_item1'] = strip_symbols(
                        comment)
                if Constants.INPUT_MAKE_SIZE2 in comment:
                    parsed_io['input']['sizeof_stream_item2'] = strip_symbols(
                        comment)
            elif parsed_io['input']['signature'] is Constants.MAKE3 and not None:
                if Constants.INPUT_MAKE_SIZE1 in comment:
                    parsed_io['input']['sizeof_stream_item1'] = strip_symbols(
                        comment)
                if Constants.INPUT_MAKE_SIZE2 in comment:
                    parsed_io['input']['sizeof_stream_item2'] = strip_symbols(
                        comment)
                if Constants.INPUT_MAKE_SIZE3 in comment:
                    parsed_io['input']['sizeof_stream_item3'] = strip_symbols(
                        comment)
            elif parsed_io['input']['signature'] is Constants.MAKEV and not None:
                if Constants.INPUT_MAKEV_SIZE in comment:
                    parsed_io['input']['sizeof_stream_items'] = strip_symbols(
                        comment)

            if Constants.OUTPUT_SIG in comment:
                parsed_io['output']['signature'] = strip_symbols(comment)
            if Constants.OUTPUT_MAX in comment:
                parsed_io['output']['max_streams'] = strip_symbols(comment)
            if Constants.OUTPUT_MIN in comment:
                parsed_io['output']['min_streams'] = strip_symbols(comment)
            if parsed_io['output']['signature'] is Constants.MAKE and not None:
                if Constants.OUTPUT_MAKE_SIZE in comment:
                    parsed_io['output']['sizeof_stream_item'] = strip_symbols(
                        comment)
            elif parsed_io['output']['signature'] is Constants.MAKE2:
                if Constants.OUTPUT_MAKE_SIZE1 in comment:
                    parsed_io['output']['sizeof_stream_item1'] = strip_symbols(
                        comment)
                if Constants.OUTPUT_MAKE_SIZE2 in comment:
                    parsed_io['output']['sizeof_stream_item2'] = strip_symbols(
                        comment)
            elif parsed_io['output']['signature'] is Constants.MAKE3 and not None:
                if Constants.OUTPUT_MAKE_SIZE1 in comment:
                    parsed_io['output']['sizeof_stream_item1'] = strip_symbols(
                        comment)
                if Constants.OUTPUT_MAKE_SIZE2 in comment:
                    parsed_io['output']['sizeof_stream_item2'] = strip_symbols(
                        comment)
                if Constants.OUTPUT_MAKE_SIZE3 in comment:
                    parsed_io['output']['sizeof_stream_item3'] = strip_symbols(
                        comment)
            elif parsed_io['output']['signature'] is Constants.MAKEV and not None:
                if Constants.OUTPUT_MAKEV_SIZE in comment:
                    parsed_io['output']['sizeof_stream_items'] = strip_symbols(
                        comment)

            if Constants.INPUT_PORT in comment:
                if Constants.EXCLAMATION in comment:
                    _suppress_input = True
                if strip_symbols(comment):
                    message_port['input'] = strip_symbols(comment).split(', ')
            if Constants.OUTPUT_PORT in comment:
                if Constants.EXCLAMATION in comment:
                    _suppress_output = True
                if strip_symbols(comment):
                    message_port['output'] = strip_symbols(comment).split(', ')
    validate_message_port(
        self, temp_parsed_data['message_port'], _suppress_input, _suppress_output)
    self.parsed_data['io_signature'] = temp_parsed_data['io_signature']


def add_comments(self):
    """
    function to add special blocktool comments
    in the public header
    """
    _index = None
    lines = []
    parsed_io = self.parsed_data['io_signature']
    message_port = self.parsed_data['message_port']
    with open(self.target_file, 'r') as header:
        lines = header.readlines()
        for line in lines:
            if Constants.BLOCKTOOL in line:
                _index = lines.index(line)
    if _index is None:
        with open(self.target_file, 'a') as header:
            header.write('\n')
            header.write('/* ' + Constants.BLOCKTOOL + '\n')
            header.write('input_signature: ' +
                         parsed_io['input']['signature'] + '\n')
            header.write('input_min_streams: ' +
                         parsed_io['input']['min_streams'] + '\n')
            header.write('input_max_streams: ' +
                         parsed_io['input']['max_streams'] + '\n')
            if parsed_io['input']['signature'] is Constants.MAKE:
                header.write('input_sizeof_stream_item: ' +
                             parsed_io['input']['sizeof_stream_item'] + '\n')
            elif parsed_io['input']['signature'] is Constants.MAKE2:
                header.write('input_sizeof_stream_item1: ' +
                             parsed_io['input']['sizeof_stream_item1'] + '\n')
                header.write('input_sizeof_stream_item2: ' +
                             parsed_io['input']['sizeof_stream_item2'] + '\n')
            elif parsed_io['input']['signature'] is Constants.MAKE3:
                header.write('input_sizeof_stream_item1: ' +
                             parsed_io['input']['sizeof_stream_item1'] + '\n')
                header.write('input_sizeof_stream_item2: ' +
                             parsed_io['input']['sizeof_stream_item2'] + '\n')
                header.write('input_sizeof_stream_item3: ' +
                             parsed_io['input']['sizeof_stream_item3'] + '\n')
            elif parsed_io['input']['signature'] is Constants.MAKEV:
                header.write('input_sizeof_stream_item: ' +
                             parsed_io['input']['sizeof_stream_items'] + '\n')
            header.write('output_signature: ' +
                         parsed_io['output']['signature'] + '\n')
            header.write('output_min_streams: ' +
                         parsed_io['output']['min_streams'] + '\n')
            header.write('output_max_streams: ' +
                         parsed_io['output']['max_streams'] + '\n')
            if parsed_io['output']['signature'] is Constants.MAKE:
                header.write('output_sizeof_stream_item: ' +
                             parsed_io['output']['sizeof_stream_item'] + '\n')
            elif parsed_io['output']['signature'] is Constants.MAKE2:
                header.write('output_sizeof_stream_item1: ' +
                             parsed_io['output']['sizeof_stream_item1'] + '\n')
                header.write('output_sizeof_stream_item2: ' +
                             parsed_io['output']['sizeof_stream_item2'] + '\n')
            elif parsed_io['output']['signature'] is Constants.MAKE3:
                header.write('output_sizeof_stream_item1: ' +
                             parsed_io['output']['sizeof_stream_item1'] + '\n')
                header.write('output_sizeof_stream_item2: ' +
                             parsed_io['output']['sizeof_stream_item2'] + '\n')
                header.write('output_sizeof_stream_item3: ' +
                             parsed_io['output']['sizeof_stream_item3'] + '\n')
            elif parsed_io['output']['signature'] is Constants.MAKEV:
                header.write('output_sizeof_stream_item: ' +
                             parsed_io['output']['sizeof_stream_items'] + '\n')

            if message_port['input']:
                header.write('message_input: ' +
                             ', '.join(message_port['input']) + '\n')
            else:
                header.write('message_input: ' + '\n')
            if message_port['output']:
                header.write('message_output: ' +
                             ', '.join(message_port['output']) + '\n')
            else:
                header.write('message_output: ' + '\n')
            header.write(Constants.END_BLOCKTOOL + '*/' + '\n')



#!/usr/bin/env python
#
# Copyright 2019 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#


from gnuradio import gr, gr_unittest, analog, blocks


class test_fm_emph(gr_unittest.TestCase):

    def setUp(self):
        self.tb = gr.top_block()

    def tearDown(self):
        self.tb = None

    def test_001(self):
        # Confirm that we can instantiate and run an FM preemphasis block
        tb = self.tb

        src = analog.sig_source_f(48000, analog.GR_COS_WAVE, 5000.0, 1.0)
        op = analog.fm_preemph(fs=48000, tau=75e-6, fh=-1.0)
        head = blocks.head(gr.sizeof_float, 100)
        dst = blocks.vector_sink_f()

        tb.connect(src, op)
        tb.connect(op, head)
        tb.connect(head, dst)
        tb.run()

    def test_002(self):
        # Confirm that we can instantiate and run an FM deemphasis block
        tb = self.tb

        src = analog.sig_source_f(48000, analog.GR_COS_WAVE, 5000.0, 1.0)
        op = analog.fm_deemph(fs=48000, tau=75e-6)
        head = blocks.head(gr.sizeof_float, 100)
        dst = blocks.vector_sink_f()

        tb.connect(src, op)
        tb.connect(op, head)
        tb.connect(head, dst)
        tb.run()


if __name__ == '__main__':
    gr_unittest.run(test_fm_emph)



#!/usr/bin/env python
#
# Copyright 2012-2014 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#


from gnuradio import gr, gr_unittest, fft, filter, blocks

import math


def sig_source_c(samp_rate, freq, amp, N):
    t = [float(x) / samp_rate for x in range(N)]
    y = [math.cos(2. * math.pi * freq * x) +
         1j * math.sin(2. * math.pi * freq * x) for x in t]
    return y


class test_pfb_synthesizer(gr_unittest.TestCase):

    def setUp(self):
        self.tb = gr.top_block()

    def tearDown(self):
        self.tb = None

    def test_000(self):
        N = 10000         # number of samples to use
        M = 5            # Number of channels
        fs = 1000        # baseband sampling rate
        ofs = M * fs       # input samp rate to decimator

        taps = filter.firdes.low_pass_2(
            M,
            ofs,
            fs / 2,
            fs / 10,
            attenuation_dB=80,
            window=fft.window.WIN_BLACKMAN_hARRIS)

        signals = list()
        freqs = [0, 100, 200, -200, -100]
        for i in range(len(freqs)):
            data = sig_source_c(fs, freqs[i], 1, N)
            signals.append(blocks.vector_source_c(data))

        pfb = filter.pfb_synthesizer_ccf(M, taps)
        snk = blocks.vector_sink_c()

        for i in range(M):
            self.tb.connect(signals[i], (pfb, i))

        self.tb.connect(pfb, snk)

        self.tb.run()

        # Adjusted phase rotations for data
        p0 = 0
        p1 = 2 * math.pi * 1.0 / M
        p2 = 2 * math.pi * 2.0 / M
        p3 = 2 * math.pi * 3.0 / M
        p4 = 2 * math.pi * 4.0 / M

        Ntest = 1000
        L = len(snk.data())
        t = [float(x) / ofs for x in range(L)]

        # Create known data as sum of complex sinusoids at freqs
        # of the output channels.
        freqs = [-2200, -1100, 0, 1100, 2200]
        expected_data = len(t) * [0, ]
        for i in range(len(t)):
            expected_data[i] = math.cos(2. * math.pi * freqs[0] * t[i] + p3) + \
                1j * math.sin(2. * math.pi * freqs[0] * t[i] + p3) + \
                math.cos(2. * math.pi * freqs[1] * t[i] + p4) + \
                1j * math.sin(2. * math.pi * freqs[1] * t[i] + p4) + \
                math.cos(2. * math.pi * freqs[2] * t[i] + p0) + \
                1j * math.sin(2. * math.pi * freqs[2] * t[i] + p0) + \
                math.cos(2. * math.pi * freqs[3] * t[i] + p1) + \
                1j * math.sin(2. * math.pi * freqs[3] * t[i] + p1) + \
                math.cos(2. * math.pi * freqs[4] * t[i] + p2) + \
                1j * math.sin(2. * math.pi * freqs[4] * t[i] + p2)
        dst_data = snk.data()

        self.assertComplexTuplesAlmostEqual(expected_data[2000:2000 + Ntest],
                                            dst_data[2000:2000 + Ntest], 4)


if __name__ == '__main__':
    gr_unittest.run(test_pfb_synthesizer)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#


from gnuradio import gr, gr_unittest
from gnuradio import blocks, zeromq
from gnuradio import eng_notation
import threading


class qa_zeromq_pub (gr_unittest.TestCase):

    def setUp(self):
        self.tb = gr.top_block()

    def tearDown(self):
        self.tb = None

    def test_001(self):
        vlen = 10
        self.rx_data = None
        src_data = list(range(vlen)) * 100
        src = blocks.vector_source_f(src_data, False, vlen)
        zeromq_pub_sink = zeromq.pub_sink(
            gr.sizeof_float, vlen, "tcp://127.0.0.1:0")
        address = zeromq_pub_sink.last_endpoint()
        self.tb.connect(src, zeromq_pub_sink)
        self.probe_manager = zeromq.probe_manager()
        self.probe_manager.add_socket(address, 'float32', self.recv_data)
        zmq_pull_t = threading.Thread(target=self.probe_manager.watcher)
        zmq_pull_t.daemon = True
        zmq_pull_t.start()
        self.tb.run()
        zmq_pull_t.join(6.0)
        # Check to see if we timed out
        self.assertFalse(zmq_pull_t.is_alive())
        self.assertFloatTuplesAlmostEqual(self.rx_data, src_data)

    def recv_data(self, data):
        self.rx_data = data


if __name__ == '__main__':
    gr_unittest.run(qa_zeromq_pub)



import time


class StopWatch(object):
    '''
    Tool for tracking/profiling application execution. Once initialized, this tracks elapsed
    time between "laps" which can be given a name and accessed at a later point.
    '''

    def __init__(self):
        self._laps = {}
        self._previous = time.time()

    def lap(self, name):
        # Do as little as possible since this is timing things.
        # Save the previous and current time, then overwrite the previous
        lap = (self._previous, time.time())
        self._previous = lap[1]
        self._laps[name] = lap

    def elapsed(self, name):
        # If the lap isn't defined, this should throw a key error
        # Don't worry about catching it here
        start, stop = self._laps[name]
        return stop - start



#!/usr/bin/env python
#
# Copyright 2005,2007,2010,2012,2013 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#


from gnuradio import gr, gr_unittest, blocks


class test_nlog10(gr_unittest.TestCase):

    def setUp(self):
        self.tb = gr.top_block()

    def tearDown(self):
        self.tb = None

    def test_001(self):
        src_data = (1, 10, 100, 1000, 10000, 100000)
        expected_result = (0, 10, 20, 30, 40, 50)
        src = blocks.vector_source_f(src_data)
        op = blocks.nlog10_ff(10)
        dst = blocks.vector_sink_f()
        self.tb.connect(src, op, dst)
        self.tb.run()
        result_data = dst.data()
        self.assertFloatTuplesAlmostEqual(expected_result, result_data, 4)


if __name__ == '__main__':
    gr_unittest.run(test_nlog10)



#!/usr/bin/env python
#
# Copyright 2020 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#

from gnuradio import gr, gr_unittest
from gnuradio import blocks

import random
import numpy
from gnuradio import digital
from gnuradio import channels


class qa_meas_evm_cc(gr_unittest.TestCase):

    def setUp(self):
        random.seed(987654)
        self.tb = gr.top_block()
        self.num_data = num_data = 1000

    def tearDown(self):
        self.tb = None

    def test_qpsk(self):
        # set up fg
        expected_result = list(numpy.zeros((self.num_data,)))

        self.cons = cons = digital.constellation_qpsk().base()
        self.data = data = [random.randrange(
            len(cons.points())) for x in range(self.num_data)]
        self.symbols = symbols = numpy.squeeze(
            [cons.map_to_points_v(i) for i in data])

        evm = digital.meas_evm_cc(cons, digital.evm_measurement_t.EVM_PERCENT)
        vso = blocks.vector_source_c(symbols, False, 1, [])
        # mc = blocks.multiply_const_cc(3.0+2.0j)
        vsi = blocks.vector_sink_f()

        self.tb.connect(vso, evm, vsi)
        self.tb.run()
        # check data

        output_data = vsi.data()
        self.assertEqual(expected_result, output_data)

    def test_qpsk_nonzeroevm(self):
        # set up fg
        expected_result = list(numpy.zeros((self.num_data,)))

        self.cons = cons = digital.constellation_qpsk().base()
        self.data = data = [random.randrange(
            len(cons.points())) for x in range(self.num_data)]
        self.symbols = symbols = numpy.squeeze(
            [cons.map_to_points_v(i) for i in data])

        evm = digital.meas_evm_cc(cons, digital.evm_measurement_t.EVM_PERCENT)
        vso = blocks.vector_source_c(symbols, False, 1, [])
        mc = blocks.multiply_const_cc(3.0 + 2.0j)
        vsi = blocks.vector_sink_f()

        self.tb.connect(vso, mc, evm, vsi)
        self.tb.run()
        # check data

        output_data = vsi.data()
        self.assertNotEqual(expected_result, output_data)

    def test_qpsk_channel(self):
        upper_bound = list(50.0 * numpy.ones((self.num_data,)))
        lower_bound = list(0.0 * numpy.zeros((self.num_data,)))

        self.cons = cons = digital.constellation_qpsk().base()
        self.data = data = [random.randrange(
            len(cons.points())) for x in range(self.num_data)]
        self.symbols = symbols = numpy.squeeze(
            [cons.map_to_points_v(i) for i in data])

        chan = channels.channel_model(
            noise_voltage=0.1,
            frequency_offset=0.0,
            epsilon=1.0,
            taps=[1.0 + 0.0j],
            noise_seed=0,
            block_tags=False)

        evm = digital.meas_evm_cc(cons, digital.evm_measurement_t.EVM_PERCENT)
        vso = blocks.vector_source_c(symbols, False, 1, [])
        mc = blocks.multiply_const_cc(3.0 + 2.0j)
        vsi = blocks.vector_sink_f()

        self.tb.connect(vso, chan, evm, vsi)
        self.tb.run()
        # check data

        output_data = vsi.data()
        self.assertLess(output_data, upper_bound)
        self.assertGreater(output_data, lower_bound)

    def test_qam16_channel(self):
        upper_bound = list(50.0 * numpy.ones((self.num_data,)))
        lower_bound = list(0.0 * numpy.zeros((self.num_data,)))

        self.cons = cons = digital.constellation_16qam().base()
        self.data = data = [random.randrange(
            len(cons.points())) for x in range(self.num_data)]
        self.symbols = symbols = numpy.squeeze(
            [cons.map_to_points_v(i) for i in data])

        chan = channels.channel_model(
            noise_voltage=0.1,
            frequency_offset=0.0,
            epsilon=1.0,
            taps=[1.0 + 0.0j],
            noise_seed=0,
            block_tags=False)

        evm = digital.meas_evm_cc(cons, digital.evm_measurement_t.EVM_PERCENT)
        vso = blocks.vector_source_c(symbols, False, 1, [])
        mc = blocks.multiply_const_cc(3.0 + 2.0j)
        vsi = blocks.vector_sink_f()

        self.tb.connect(vso, chan, evm, vsi)
        self.tb.run()
        # check data

        output_data = vsi.data()
        self.assertLess(output_data, upper_bound)
        self.assertGreater(output_data, lower_bound)


if __name__ == '__main__':
    gr_unittest.run(qa_meas_evm_cc)



#!/usr/bin/env python
# Copyright 2012-2014 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#


from gnuradio import gr, gr_unittest, blocks, digital
import pmt


class qa_crc32_bb(gr_unittest.TestCase):
    def setUp(self):
        self.tb = gr.top_block()
        self.tsb_key = "length"

    def tearDown(self):
        self.tb = None

    def test_001_crc_len(self):
        """ Make sure the output of a CRC set is 4 bytes longer than the input. """
        data = list(range(16))
        src = blocks.vector_source_b(data)
        crc = digital.crc32_bb(False, self.tsb_key)
        sink = blocks.tsb_vector_sink_b(tsb_key=self.tsb_key)
        self.tb.connect(
            src,
            blocks.stream_to_tagged_stream(gr.sizeof_char, 1,
                                           len(data), self.tsb_key), crc, sink)
        self.tb.run()
        # Check that the packets before crc_check are 4 bytes longer that the
        # input.
        self.assertEqual(len(data) + 4, len(sink.data()[0]))

    def test_002_crc_equal(self):
        """ Go through CRC set / CRC check and make sure the output
        is the same as the input. """
        data = [0, 1, 2, 3, 4, 5, 6, 7, 8]
        src = blocks.vector_source_b(data)
        crc = digital.crc32_bb(False, self.tsb_key)
        crc_check = digital.crc32_bb(True, self.tsb_key)
        sink = blocks.tsb_vector_sink_b(tsb_key=self.tsb_key)
        self.tb.connect(src,
                        blocks.stream_to_tagged_stream(
                            gr.sizeof_char, 1, len(data), self.tsb_key), crc,
                        crc_check, sink)
        self.tb.run()
        # Check that the packets after crc_check are the same as input.
        self.assertEqual(data, sink.data()[0])

    def test_003_crc_correct_lentag(self):
        tag_name = "length"
        pack_len = 8
        packets = list(range(pack_len * 2))
        tag1 = gr.tag_t()
        tag1.offset = 0
        tag1.key = pmt.string_to_symbol(tag_name)
        tag1.value = pmt.from_long(pack_len)
        tag2 = gr.tag_t()
        tag2.offset = pack_len
        tag2.key = pmt.string_to_symbol(tag_name)
        tag2.value = pmt.from_long(pack_len)
        testtag1 = gr.tag_t()
        testtag1.offset = 1
        testtag1.key = pmt.string_to_symbol("tag1")
        testtag1.value = pmt.from_long(0)
        testtag2 = gr.tag_t()
        testtag2.offset = pack_len
        testtag2.key = pmt.string_to_symbol("tag2")
        testtag2.value = pmt.from_long(0)
        testtag3 = gr.tag_t()
        testtag3.offset = len(packets) - 1
        testtag3.key = pmt.string_to_symbol("tag3")
        testtag3.value = pmt.from_long(0)
        src = blocks.vector_source_b(packets, False, 1,
                                     (testtag1, testtag2, testtag3))
        crc = digital.crc32_bb(False, self.tsb_key)
        sink = blocks.tsb_vector_sink_b(tsb_key=self.tsb_key)
        self.tb.connect(src,
                        blocks.stream_to_tagged_stream(
                            gr.sizeof_char, 1, pack_len, self.tsb_key), crc,
                        sink)
        self.tb.run()
        self.assertEqual(len(sink.data()), 2)
        self.assertEqual(len(sink.data()[0]), (pack_len + 4))
        self.assertEqual(len(sink.data()[1]), (pack_len + 4))
        correct_offsets = {'tag1': 1, 'tag2': 12, 'tag3': 19}
        tags_found = {'tag1': False, 'tag2': False, 'tag3': False}
        for tag in sink.tags():
            key = pmt.symbol_to_string(tag.key)
            if key in list(correct_offsets.keys()):
                tags_found[key] = True
                self.assertEqual(correct_offsets[key], tag.offset)
        self.assertTrue(all(tags_found.values()))

    def test_004_fail(self):
        """ Corrupt the data and make sure it fails CRC test. """
        data = [0, 1, 2, 3, 4, 5, 6, 7]
        src = blocks.vector_source_b(data)
        crc = digital.crc32_bb(False, self.tsb_key)
        crc_check = digital.crc32_bb(True, self.tsb_key)
        corruptor = blocks.add_const_bb(1)
        sink = blocks.tsb_vector_sink_b(tsb_key=self.tsb_key)
        self.tb.connect(src,
                        blocks.stream_to_tagged_stream(
                            gr.sizeof_char, 1, len(data), self.tsb_key), crc,
                        corruptor, crc_check, sink)
        self.tb.run()
        # crc_check will drop invalid packets
        self.assertEqual(len(sink.data()), 0)

    def test_005_tag_propagation(self):
        """ Make sure tags on the CRC aren't lost. """
        # Data with precalculated CRC
        data = [0, 1, 2, 3, 4, 5, 6, 7, 8, 2, 67, 225, 188]
        testtag = gr.tag_t()
        testtag.offset = len(data) - 1
        testtag.key = pmt.string_to_symbol('tag1')
        testtag.value = pmt.from_long(0)
        src = blocks.vector_source_b(data, False, 1, (testtag, ))
        crc_check = digital.crc32_bb(True, self.tsb_key)
        sink = blocks.tsb_vector_sink_b(tsb_key=self.tsb_key)
        self.tb.connect(src,
                        blocks.stream_to_tagged_stream(
                            gr.sizeof_char, 1, len(data), self.tsb_key),
                        crc_check, sink)
        self.tb.run()
        self.assertEqual([len(data) - 5, ], [
            tag.offset for tag in sink.tags()
            if pmt.symbol_to_string(tag.key) == 'tag1'
        ])

    # NOTE: What follows are the same tests as before but with the packed flag
    # set to False

    def test_006_crc_len(self):
        """ Make sure the output of a CRC set is 32 (unpacked) bytes longer than the input. """
        data = list(range(16))
        src = blocks.vector_source_b(data)
        crc = digital.crc32_bb(False, self.tsb_key, False)
        sink = blocks.tsb_vector_sink_b(tsb_key=self.tsb_key)
        self.tb.connect(
            src,
            blocks.stream_to_tagged_stream(gr.sizeof_char, 1,
                                           len(data), self.tsb_key), crc, sink)
        self.tb.run()
        # Check that the packets before crc_check are 4 bytes longer that the
        # input.
        self.assertEqual(len(data) + 32, len(sink.data()[0]))

    def test_007_crc_equal(self):
        """ Go through CRC set / CRC check and make sure the output
        is the same as the input. """
        data = [0, 1, 2, 3, 4, 5, 6, 7, 8]
        src = blocks.vector_source_b(data)
        crc = digital.crc32_bb(False, self.tsb_key, False)
        crc_check = digital.crc32_bb(True, self.tsb_key, False)
        sink = blocks.tsb_vector_sink_b(tsb_key=self.tsb_key)
        self.tb.connect(src,
                        blocks.stream_to_tagged_stream(
                            gr.sizeof_char, 1, len(data), self.tsb_key), crc,
                        crc_check, sink)
        self.tb.run()
        # Check that the packets after crc_check are the same as input.
        self.assertEqual(data, sink.data()[0])

    def test_002_crc_equal_unpacked(self):
        """ Test unpacked operation with packed operation
        """
        data = [0, 1, 2, 3, 4, 5, 6, 7, 8]
        src = blocks.vector_source_b(data)
        unpack1 = blocks.repack_bits_bb(
            8, 1, self.tsb_key, False, gr.GR_LSB_FIRST)
        unpack2 = blocks.repack_bits_bb(
            8, 1, self.tsb_key, False, gr.GR_LSB_FIRST)
        crc_unpacked = digital.crc32_bb(False, self.tsb_key, False)
        crc_packed = digital.crc32_bb(False, self.tsb_key, True)
        sink1 = blocks.tsb_vector_sink_b(tsb_key=self.tsb_key)
        sink2 = blocks.tsb_vector_sink_b(tsb_key=self.tsb_key)

        self.tb.connect(src,
                        blocks.stream_to_tagged_stream(
                            gr.sizeof_char, 1, len(data), self.tsb_key),
                        crc_packed, unpack1, sink1)
        self.tb.connect(src,
                        blocks.stream_to_tagged_stream(
                            gr.sizeof_char, 1, len(data), self.tsb_key),
                        unpack2, crc_unpacked, sink2)
        self.tb.run()
        self.assertEqual(sink1.data(), sink2.data())

    def test_003_crc_equal_unpacked(self):
        """ Test unpacked operation with packed operation
        """
        data = list(range(35))
        src = blocks.vector_source_b(data)
        unpack1 = blocks.repack_bits_bb(8, 1, self.tsb_key, False,
                                        gr.GR_LSB_FIRST)
        unpack2 = blocks.repack_bits_bb(8, 1, self.tsb_key, False,
                                        gr.GR_LSB_FIRST)
        crc_unpacked = digital.crc32_bb(False, self.tsb_key, False)
        crc_packed = digital.crc32_bb(False, self.tsb_key, True)
        sink1 = blocks.tsb_vector_sink_b(tsb_key=self.tsb_key)
        sink2 = blocks.tsb_vector_sink_b(tsb_key=self.tsb_key)

        self.tb.connect(src,
                        blocks.stream_to_tagged_stream(
                            gr.sizeof_char, 1, len(data), self.tsb_key),
                        crc_packed, unpack1, sink1)
        self.tb.connect(src,
                        blocks.stream_to_tagged_stream(
                            gr.sizeof_char, 1, len(data), self.tsb_key),
                        unpack2, crc_unpacked, sink2)
        self.tb.run()
        self.assertEqual(sink1.data(), sink2.data())

    def test_008_crc_correct_lentag(self):
        tag_name = "length"
        pack_len = 8
        packets = list(range(pack_len * 2))
        tag1 = gr.tag_t()
        tag1.offset = 0
        tag1.key = pmt.string_to_symbol(tag_name)
        tag1.value = pmt.from_long(pack_len)
        tag2 = gr.tag_t()
        tag2.offset = pack_len
        tag2.key = pmt.string_to_symbol(tag_name)
        tag2.value = pmt.from_long(pack_len)
        testtag1 = gr.tag_t()
        testtag1.offset = 1
        testtag1.key = pmt.string_to_symbol("tag1")
        testtag1.value = pmt.from_long(0)
        testtag2 = gr.tag_t()
        testtag2.offset = pack_len
        testtag2.key = pmt.string_to_symbol("tag2")
        testtag2.value = pmt.from_long(0)
        testtag3 = gr.tag_t()
        testtag3.offset = len(packets) - 1
        testtag3.key = pmt.string_to_symbol("tag3")
        testtag3.value = pmt.from_long(0)
        src = blocks.vector_source_b(packets, False, 1,
                                     (testtag1, testtag2, testtag3))
        crc = digital.crc32_bb(False, self.tsb_key, False)
        sink = blocks.tsb_vector_sink_b(tsb_key=self.tsb_key)
        self.tb.connect(src,
                        blocks.stream_to_tagged_stream(
                            gr.sizeof_char, 1, pack_len, self.tsb_key), crc,
                        sink)
        self.tb.run()
        self.assertEqual(len(sink.data()), 2)
        self.assertEqual(len(sink.data()[0]), (pack_len + 32))
        self.assertEqual(len(sink.data()[1]), (pack_len + 32))
        correct_offsets = {'tag1': 1, 'tag2': 8 + 32, 'tag3': 15 + 32}
        tags_found = {'tag1': False, 'tag2': False, 'tag3': False}
        for tag in sink.tags():
            key = pmt.symbol_to_string(tag.key)
            if key in list(correct_offsets.keys()):
                tags_found[key] = True
                self.assertEqual(correct_offsets[key], tag.offset)
        self.assertTrue(all(tags_found.values()))

    def test_009_fail(self):
        """ Corrupt the data and make sure it fails CRC test. """
        data = (0, 1, 2, 3, 4, 5, 6, 7)
        src = blocks.vector_source_b(data)
        crc = digital.crc32_bb(False, self.tsb_key, False)
        crc_check = digital.crc32_bb(True, self.tsb_key, False)
        corruptor = blocks.add_const_bb(1)
        sink = blocks.tsb_vector_sink_b(tsb_key=self.tsb_key)
        self.tb.connect(src,
                        blocks.stream_to_tagged_stream(
                            gr.sizeof_char, 1, len(data), self.tsb_key), crc,
                        corruptor, crc_check, sink)
        self.tb.run()
        # crc_check will drop invalid packets
        self.assertEqual(len(sink.data()), 0)

    def test_0010_tag_propagation(self):
        """ Make sure tags on the CRC aren't lost. """
        # Data with precalculated CRC
        data = (0, 0, 0, 0, 0, 0, 0, 0,
                1, 0, 0, 0, 0, 0, 0, 0,
                0, 1, 0, 0, 0, 0, 0, 0,
                1, 1, 0, 0, 0, 0, 0, 0,
                0, 0, 1, 0, 0, 0, 0, 0,
                1, 0, 1, 0, 0, 0, 0, 0,
                0, 1, 1, 0, 0, 0, 0, 0,
                1, 1, 1, 0, 0, 0, 0, 0,
                0, 0, 0, 1, 0, 0, 0, 0,
                0, 1, 0, 0, 0, 0, 0, 0,
                1, 1, 0, 0, 0, 0, 1, 0,
                1, 0, 0, 0, 0, 1, 1, 1,
                0, 0, 1, 1, 1, 1, 0, 1)
        testtag = gr.tag_t()
        testtag.offset = len(data) - 1
        testtag.key = pmt.string_to_symbol('tag1')
        testtag.value = pmt.from_long(0)
        src = blocks.vector_source_b(data, False, 1, (testtag, ))
        crc_check = digital.crc32_bb(True, self.tsb_key, False)
        sink = blocks.tsb_vector_sink_b(tsb_key=self.tsb_key)
        self.tb.connect(src,
                        blocks.stream_to_tagged_stream(
                            gr.sizeof_char, 1, len(data), self.tsb_key),
                        crc_check, sink)
        self.tb.run()
        self.assertEqual([len(data) - 33, ], [
            tag.offset for tag in sink.tags()
            if pmt.symbol_to_string(tag.key) == 'tag1'
        ])


if __name__ == '__main__':
    gr_unittest.run(qa_crc32_bb)



#!/usr/bin/env python
#
# Copyright 2004,2010,2012 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#


import math

from gnuradio import gr, gr_unittest, trellis, digital, analog, blocks

fsm_args = {"awgn1o2_4": [2, 4, 4,
                          [0, 2, 0, 2, 1, 3, 1, 3],
                          [0, 3, 3, 0, 1, 2, 2, 1],
                          ],
            "rep2": [2, 1, 4, [0, 0], [0, 3]],
            "nothing": [2, 1, 2, [0, 0], [0, 1]],
            }

constells = {2: digital.constellation_bpsk(),
             4: digital.constellation_qpsk(),
             }


class test_trellis (gr_unittest.TestCase):

    def test_001_fsm(self):
        f = trellis.fsm(*fsm_args["awgn1o2_4"])
        self.assertEqual(
            fsm_args["awgn1o2_4"], [
                f.I(), f.S(), f.O(), f.NS(), f.OS()])

    def test_002_fsm(self):
        f = trellis.fsm(*fsm_args["awgn1o2_4"])
        g = trellis.fsm(f)
        self.assertEqual((g.I(), g.S(), g.O(), g.NS(), g.OS()),
                         (f.I(), f.S(), f.O(), f.NS(), f.OS()))

    def test_003_fsm(self):
        """ Test to make sure fsm works with a single state fsm."""
        # Just checking that it initializes properly.
        f = trellis.fsm(*fsm_args["rep2"])

    def test_011_viterbi(self):
        """
        Runs some coding/decoding tests with a few different FSM
        specs.
        """
        for name, args in list(fsm_args.items()):
            constellation = constells[args[2]]
            fsms = trellis.fsm(*args)
            noise = 0.1
            tb = trellis_tb(constellation, fsms, noise)
            tb.run()
            # Make sure all packets successfully transmitted.
            self.assertEqual(tb.dst.ntotal(), tb.dst.nright())

    def test_012_viterbi_combined(self):
        ftypes = ["ss", "si", "ib", "is", "ii", "fb", "fs", "fi", "cb", "cs", "ci"]
        for ftype in ftypes:
            tb = trellis_comb_tb(ftype)
            tb.run()

    def test_021_pccc_encoder(self):
        ftypes = ["bb", "bs", "bi", "ss", "si", "ii"]
        for ftype in ftypes:
            tb = trellis_pccc_encoder_tb(ftype)

    def test_022_pccc_decoder(self):
        ftypes = ["b", "s", "i"]
        for ftype in ftypes:
            tb = trellis_pccc_decoder_tb(ftype)
            tb.run()

    def test_023_pccc_decoder_combined(self):
        ftypes = ["fb", "fs", "fi", "cb", "cs", "ci"]
        for ftype in ftypes:
            tb = trellis_pccc_decoder_combined_tb(ftype)
            tb.run()


class trellis_comb_tb(gr.top_block):
    def test_001_sccc_encoder(self):
        ftypes = ["bb", "bs", "bi", "ss", "si", "ii"]
        for ftype in ftypes:
            tb = trellis_sccc_encoder_tb(ftype)
            tb.run()


class trellis_sccc_encoder_tb(gr.top_block):
    def test_001_sccc_decoder_combined(self):
        ftypes = ["fb", "fs", "fi", "cb", "cs", "ci"]
        for ftype in ftypes:
            tb = trellis_sccc_decoder_combined_tb(ftype)
            tb.run()


class trellis_sccc_decoder_combined_tb(gr.top_block):
    """
    A simple top block for use testing gr-trellis.
    """

    def __init__(self, ftype):
        super(trellis_sccc_decoder_combined_tb, self).__init__()
        func = eval("trellis.sccc_decoder_combined_" + ftype)
        dsttype = gr.sizeof_int
        if ftype[1] == "b":
            dsttype = gr.sizeof_char
        elif ftype[1] == "s":
            dsttype = gr.sizeof_short
        elif ftype[1] == "i":
            dsttype = gr.sizeof_int
        src_func = eval("blocks.vector_source_" + ftype[0])
        data = [1 * 200]
        src = src_func(data)
        self.dst = blocks.null_sink(dsttype * 1)
        constellation = [1, 1, 1, 1, 1, -1, 1, -1, 1, 1, -1, -1, -1, 1, 1, -1, 1, -1, -1, -1, 1, -1, -1, -1]
        vbc = func(trellis.fsm(), 0, -1, trellis.fsm(1, 3, [91, 121, 117]), 0, -1, trellis.interleaver(), 10, 10,
                   trellis.TRELLIS_MIN_SUM, 2, constellation, digital.TRELLIS_EUCLIDEAN, 1.0)

        ##################################################
        # Connections
        ##################################################

        self.connect((src, 0), (vbc, 0))
        self.connect((vbc, 0), (self.dst, 0))


class trellis_pccc_encoder_tb(gr.top_block):
    def test_001_sccc_decoder(self):
        ftypes = ["b", "s", "i"]
        for ftype in ftypes:
            tb = trellis_sccc_decoder_tb(ftype)
            tb.run()


class trellis_sccc_decoder_tb(gr.top_block):
    """
    A simple top block for use testing gr-trellis.
    """

    def __init__(self, ftype):
        super(trellis_sccc_decoder_tb, self).__init__()
        func = eval("trellis.sccc_decoder_" + ftype)
        dsttype = gr.sizeof_int
        if ftype == "b":
            dsttype = gr.sizeof_char
        elif ftype == "s":
            dsttype = gr.sizeof_short
        elif ftype == "i":
            dsttype = gr.sizeof_int
        data = [1 * 200]
        src = blocks.vector_source_f(data)
        self.dst = blocks.null_sink(dsttype * 1)
        vbc = func(trellis.fsm(), 0, -1, trellis.fsm(1, 3, [91, 121, 117]),
                   0, -1, trellis.interleaver(), 10, 10, trellis.TRELLIS_MIN_SUM)
        self.connect((src, 0), (vbc, 0))
        self.connect((vbc, 0), (self.dst, 0))


class trellis_tb(gr.top_block):
    """
    A simple top block for use testing gr-trellis.
    """

    def __init__(self, constellation, f, N0=0.25, seed=-666):
        """
        constellation - a constellation object used for modulation.
        f - a finite state machine specification used for coding.
        N0 - noise level
        seed - random seed
        """
        super(trellis_tb, self).__init__()
        # packet size in bits (make it multiple of 16 so it can be packed in a
        # short)
        packet_size = 1024 * 16
        # bits per FSM input symbol
        # bits per FSM input symbol
        bitspersymbol = int(round(math.log(f.I()) / math.log(2)))
        # packet size in trellis steps
        K = packet_size // bitspersymbol

        # TX
        src = blocks.lfsr_32k_source_s()
        # packet size in shorts
        src_head = blocks.head(gr.sizeof_short, packet_size // 16)
        # unpack shorts to symbols compatible with the FSM input cardinality
        s2fsmi = blocks.packed_to_unpacked_ss(bitspersymbol, gr.GR_MSB_FIRST)
        # initial FSM state = 0
        enc = trellis.encoder_ss(f, 0)
        mod = digital.chunks_to_symbols_sc(constellation.points(), 1)

        # CHANNEL
        add = blocks.add_cc()
        noise = analog.noise_source_c(
            analog.GR_GAUSSIAN, math.sqrt(
                N0 / 2), seed)

        # RX
        # data preprocessing to generate metrics for Viterbi
        metrics = trellis.constellation_metrics_cf(
            constellation.base(), digital.TRELLIS_EUCLIDEAN)
        # Put -1 if the Initial/Final states are not set.
        va = trellis.viterbi_s(f, K, 0, -1)
        # pack FSM input symbols to shorts
        fsmi2s = blocks.unpacked_to_packed_ss(bitspersymbol, gr.GR_MSB_FIRST)
        # check the output
        self.dst = blocks.check_lfsr_32k_s()

        self.connect(src, src_head, s2fsmi, enc, mod)
        self.connect(mod, (add, 0))
        self.connect(noise, (add, 1))
        self.connect(add, metrics, va, fsmi2s, self.dst)


class trellis_pccc_decoder_tb(gr.top_block):
    """
    A simple top block for use testing gr-trellis.
    """

    def __init__(self, ftype):
        super(trellis_pccc_decoder_tb, self).__init__()
        func = eval("trellis.pccc_decoder_" + ftype)
        dsttype = gr.sizeof_int
        if ftype == "b":
            dsttype = gr.sizeof_char
        elif ftype == "s":
            dsttype = gr.sizeof_short
        elif ftype == "i":
            dsttype = gr.sizeof_int
        data = [1 * 200]
        src = blocks.vector_source_f(data)
        self.dst = blocks.null_sink(dsttype * 1)
        vbc = func(trellis.fsm(1, 3, [91, 121, 117]), 0, -1, trellis.fsm(1, 3, [91, 121, 117]),
                   0, -1, trellis.interleaver(), 10, 10, trellis.TRELLIS_MIN_SUM)
        self.connect((src, 0), (vbc, 0))
        self.connect((vbc, 0), (self.dst, 0))


class trellis_pccc_decoder_combined_tb(gr.top_block):
    """
    A simple top block for use testing gr-trellis.
    """

    def __init__(self, ftype):
        super(trellis_pccc_decoder_combined_tb, self).__init__()
        func = eval("trellis.pccc_decoder_combined_" + ftype)
        dsttype = gr.sizeof_int
        if ftype[1] == "b":
            dsttype = gr.sizeof_char
        elif ftype[1] == "s":
            dsttype = gr.sizeof_short
        elif ftype[1] == "i":
            dsttype = gr.sizeof_int
        src_func = eval("blocks.vector_source_" + ftype[0])
        data = [1 * 200]
        src = src_func(data)
        self.dst = blocks.null_sink(dsttype * 1)
        constellation = [1, 1, 1, 1, 1, -1, 1, -1, 1, 1, -1, -1, -1, 1, 1, -1, 1, -1, -1, -1, 1, -1, -1, -1]
        vbc = func(trellis.fsm(), 0, -1, trellis.fsm(), 0, -1, trellis.interleaver(), 10, 10,
                   trellis.TRELLIS_MIN_SUM, 2, constellation, digital.TRELLIS_EUCLIDEAN, 1.0)

        ##################################################
        # Connections
        ##################################################

        self.connect((src, 0), (vbc, 0))
        self.connect((vbc, 0), (self.dst, 0))


if __name__ == '__main__':
    gr_unittest.run(test_trellis)



#!/usr/bin/env python
#
# Copyright 2013 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#


from gnuradio import gr, gr_unittest, blocks

import math


class test_transcendental(gr_unittest.TestCase):

    def setUp(self):
        self.tb = gr.top_block()

    def tearDown(self):
        self.tb = None

    def test_01(self):
        tb = self.tb

        data = 100 * [0, ]
        expected_result = 100 * [1, ]

        src = blocks.vector_source_f(data, False)
        op = blocks.transcendental("cos", "float")
        dst = blocks.vector_sink_f()

        tb.connect(src, op)
        tb.connect(op, dst)
        tb.run()

        dst_data = dst.data()

        self.assertFloatTuplesAlmostEqual(expected_result, dst_data, 5)

    def test_02(self):
        tb = self.tb

        data = 100 * [3, ]
        expected_result = 100 * [math.log10(3), ]

        src = blocks.vector_source_f(data, False)
        op = blocks.transcendental("log10", "float")
        dst = blocks.vector_sink_f()

        tb.connect(src, op)
        tb.connect(op, dst)
        tb.run()

        dst_data = dst.data()

        self.assertFloatTuplesAlmostEqual(expected_result, dst_data, 5)

    def test_03(self):
        tb = self.tb

        data = 100 * [3, ]
        expected_result = 100 * [math.tanh(3), ]

        src = blocks.vector_source_f(data, False)
        op = blocks.transcendental("tanh", "float")
        dst = blocks.vector_sink_f()

        tb.connect(src, op)
        tb.connect(op, dst)
        tb.run()

        dst_data = dst.data()

        self.assertFloatTuplesAlmostEqual(expected_result, dst_data, 5)


if __name__ == '__main__':
    gr_unittest.run(test_transcendental)



#!/usr/bin/env python
#
# Copyright 2011,2013,2014 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#


from gnuradio import gr, gr_unittest, vocoder, blocks
from gnuradio.vocoder import codec2


class test_codec2_vocoder (gr_unittest.TestCase):

    def setUp(self):
        self.tb = gr.top_block()

    def tearDown(self):
        self.tb = None

    def test001_mode_2400_encoder(self):
        # compresses a
        # 16-bit 8 kHz (128 kbit/s) PCM stream
        # to 2400 bit/s
        enc = vocoder.codec2_encode_sp(codec2.MODE_2400)
        samples_per_frame = enc.relative_rate_d()
        data = samples_per_frame * (100, 200, 300, 400, 500, 600, 700, 800)
        src = blocks.vector_source_s(data)
        snk = blocks.vector_sink_b(vlen=48)

        expected_length = ((len(data) * 16) * 2400) // 128000
        self.tb.connect(src, enc, snk)
        self.tb.run()
        result = snk.data()
        self.assertEqual(expected_length, len(result))

    def test001_mode_1600_encoder(self):
        enc = vocoder.codec2_encode_sp(codec2.MODE_1600)
        samples_per_frame = enc.relative_rate_d()
        bits_per_frame = enc.output_signature().sizeof_stream_item(0)
        data = samples_per_frame * (100, 200, 300, 400, 500, 600, 700, 800)
        src = blocks.vector_source_s(data)
        snk = blocks.vector_sink_b(vlen=bits_per_frame)

        expected_length = ((len(data) * 16) * 1600) // 128000
        self.tb.connect(src, enc, snk)
        self.tb.run()
        result = snk.data()
        self.assertEqual(expected_length, len(result))

    def test001_mode_1400_encoder(self):
        enc = vocoder.codec2_encode_sp(codec2.MODE_1400)
        samples_per_frame = enc.relative_rate_d()
        bits_per_frame = enc.output_signature().sizeof_stream_item(0)
        data = samples_per_frame * (100, 200, 300, 400, 500, 600, 700, 800)
        src = blocks.vector_source_s(data)
        snk = blocks.vector_sink_b(vlen=bits_per_frame)

        expected_length = ((len(data) * 16) * 1400) // 128000
        self.tb.connect(src, enc, snk)
        self.tb.run()
        result = snk.data()
        self.assertEqual(expected_length, len(result))


if __name__ == '__main__':
    gr_unittest.run(test_codec2_vocoder)



#!/usr/bin/env python
#
# Copyright 2020 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#
"""Unit tests for ZMQ PULL Message Source block"""

import time
import zmq

from gnuradio import gr, gr_unittest, blocks, zeromq
import pmt


class qa_zeromq_pull_msg_source(gr_unittest.TestCase):
    """Unit tests for ZMQ PULL Message Source block"""

    def setUp(self):
        addr = 'tcp://127.0.0.1'
        self.context = zmq.Context()
        self.zmq_sock = self.context.socket(zmq.PUSH)
        port = self.zmq_sock.bind_to_random_port(addr)

        self.zeromq_pull_msg_source = zeromq.pull_msg_source(
            ('%s:%s' % (addr, port)), 100)
        self.message_debug = blocks.message_debug()
        self.tb = gr.top_block()
        self.tb.msg_connect(
            (self.zeromq_pull_msg_source, 'out'), (self.message_debug, 'store'))

        self.tb.start()
        time.sleep(0.1)

    def tearDown(self):
        self.tb.stop()
        self.tb.wait()
        self.zeromq_pull_msg_source = None
        self.message_debug = None
        self.tb = None
        self.zmq_sock.close()
        self.context.term()

    def test_valid_pmt(self):
        """Test receiving of valid PMT messages"""
        msg = pmt.to_pmt('test_valid_pmt')
        self.zmq_sock.send(pmt.serialize_str(msg))
        for _ in range(10):
            if self.message_debug.num_messages() > 0:
                break
            time.sleep(0.2)
        self.assertEqual(1, self.message_debug.num_messages())
        self.assertTrue(pmt.equal(msg, self.message_debug.get_message(0)))

    def test_invalid_pmt(self):
        """Test receiving of invalid PMT messages"""
        self.zmq_sock.send_string('test_invalid_pmt')
        time.sleep(0.1)
        self.assertEqual(0, self.message_debug.num_messages())


if __name__ == '__main__':
    gr_unittest.run(qa_zeromq_pull_msg_source)



#!/usr/bin/env python
#
# Copyright 2012-2014 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#


import numpy

from gnuradio import gr, gr_unittest, blocks, fft, analog, digital
import pmt


class qa_ofdm_serializer_vcc (gr_unittest.TestCase):

    def setUp(self):
        self.tb = gr.top_block()
        self.tsb_key = "ts_last"

    def tearDown(self):
        self.tb = None

    def test_001_simple(self):
        """ Standard test """
        fft_len = 16
        tx_symbols = (0, 1, 1j, 2, 3, 0, 0, 0, 0, 0, 0, 4, 5, 2j, 6, 0,
                      0, 7, 8, 3j, 9, 0, 0, 0, 0, 0, 0, 10, 4j, 11, 12, 0,
                      0, 13, 1j, 14, 15, 0, 0, 0, 0, 0, 0, 0, 0, 2j, 0, 0)
        expected_result = list(range(1, 16)) + [0, 0, 0]
        occupied_carriers = ((1, 3, 4, 11, 12, 14), (1, 2, 4, 11, 13, 14),)
        n_syms = len(tx_symbols) // fft_len
        src = blocks.vector_source_c(tx_symbols, False, fft_len)
        serializer = digital.ofdm_serializer_vcc(
            fft_len, occupied_carriers, self.tsb_key, "", 0, "", False)
        sink = blocks.tsb_vector_sink_c(tsb_key=self.tsb_key)
        self.tb.connect(
            src,
            blocks.stream_to_tagged_stream(
                gr.sizeof_gr_complex,
                fft_len,
                n_syms,
                self.tsb_key),
            serializer,
            sink)
        self.tb.run()
        self.assertEqual(sink.data()[0], expected_result)

    def test_001b_shifted(self):
        """ Same as before, but shifted, because that's the normal mode in OFDM Rx """
        fft_len = 16
        tx_symbols = (
            0, 0, 0, 0, 0, 0, 1, 2, 0, 3, 4, 5, 0, 0, 0, 0,
            0, 0, 0, 0, 6, 1j, 7, 8, 0, 9, 10, 1j, 11, 0, 0, 0,
            0, 0, 0, 0, 0, 12, 13, 14, 0, 15, 16, 17, 0, 0, 0, 0,
        )
        expected_result = list(range(18))
        occupied_carriers = ((13, 14, 15, 1, 2, 3), (-4, -2, -1, 1, 2, 4),)
        n_syms = len(tx_symbols) // fft_len
        src = blocks.vector_source_c(tx_symbols, False, fft_len)
        serializer = digital.ofdm_serializer_vcc(
            fft_len, occupied_carriers, self.tsb_key)
        sink = blocks.tsb_vector_sink_c(tsb_key=self.tsb_key)
        self.tb.connect(
            src,
            blocks.stream_to_tagged_stream(
                gr.sizeof_gr_complex,
                fft_len,
                n_syms,
                self.tsb_key),
            serializer,
            sink)
        self.tb.run()
        self.assertEqual(sink.data()[0], expected_result)

    def test_002_with_offset(self):
        """ Standard test, carrier offset """
        fft_len = 16
        tx_symbols = list(range(1, 16))
        tx_symbols = (0, 0, 1, 1j, 2, 3, 0, 0, 0, 0, 0, 0, 4, 5, 2j, 6,
                      0, 0, 7, 8, 3j, 9, 0, 0, 0, 0, 0, 0, 10, 4j, 11, 12,
                      0, 0, 13, 1j, 14, 15, 0, 0, 0, 0, 0, 0, 0, 0, 2j, 0)
        carr_offset = 1  # Compare this with tx_symbols from the previous test
        expected_result = list(range(1, 16)) + [0, 0, 0]
        occupied_carriers = ((1, 3, 4, 11, 12, 14), (1, 2, 4, 11, 13, 14),)
        n_syms = len(tx_symbols) // fft_len
        offsettag = gr.tag_t()
        offsettag.offset = 0
        offsettag.key = pmt.string_to_symbol("ofdm_sync_carr_offset")
        offsettag.value = pmt.from_long(carr_offset)
        src = blocks.vector_source_c(tx_symbols, False, fft_len, (offsettag,))
        sink = blocks.tsb_vector_sink_c(tsb_key=self.tsb_key)
        serializer = digital.ofdm_serializer_vcc(
            fft_len,
            occupied_carriers,
            self.tsb_key,
            "", 0,
            "ofdm_sync_carr_offset",
            False
        )
        self.tb.connect(
            src,
            blocks.stream_to_tagged_stream(
                gr.sizeof_gr_complex,
                fft_len,
                n_syms,
                self.tsb_key),
            serializer,
            sink)
        self.tb.run()
        self.assertEqual(sink.data()[0], expected_result)
        self.assertEqual(len(sink.tags()), 1)

    def test_003_connect(self):
        """ Connect carrier_allocator to ofdm_serializer,
            make sure output==input """
        fft_len = 8
        n_syms = 1
        occupied_carriers = ((1, 2, 6, 7),)
        pilot_carriers = ((3,), (5,))
        pilot_symbols = ((1j,), (-1j,))
        #tx_data = tuple([numpy.random.randint(0, 10) for x in range(4 * n_syms)])
        tx_data = [1, 2, 3, 4]
        src = blocks.vector_source_c(tx_data, False, 1)
        alloc = digital.ofdm_carrier_allocator_cvc(
            fft_len,
            occupied_carriers,
            pilot_carriers,
            pilot_symbols,
            (),  # No sync word
            self.tsb_key,
            True  # Output is shifted (default)
        )
        serializer = digital.ofdm_serializer_vcc(
            alloc,
            "",  # Len tag key
            0,  # Symbols skipped
            "",  # Carrier offset key
                True  # Input is shifted (default)
        )
        sink = blocks.tsb_vector_sink_c(tsb_key=self.tsb_key)
        self.tb.connect(
            src,
            blocks.stream_to_tagged_stream(
                gr.sizeof_gr_complex,
                1,
                len(tx_data),
                self.tsb_key),
            alloc,
            serializer,
            sink)
        self.tb.run()
        self.assertEqual(sink.data()[0], tx_data)

    def test_004_connect(self):
        """
        Advanced test:
        - Allocator -> IFFT -> Frequency offset -> FFT -> Serializer
        - FFT does shift (moves DC to middle)
        - Make sure input == output
        - Frequency offset is -2 carriers
        """
        fft_len = 8
        n_syms = 1
        carr_offset = -2
        freq_offset = 1.0 / fft_len * carr_offset  # Normalized frequency
        occupied_carriers = ((-2, -1, 1, 2),)
        pilot_carriers = ((-3,), (3,))
        pilot_symbols = ((1j,), (-1j,))
        tx_data = [1, 2, 3, 4]
        offsettag = gr.tag_t()
        offsettag.offset = 0
        offsettag.key = pmt.string_to_symbol("ofdm_sync_carr_offset")
        offsettag.value = pmt.from_long(carr_offset)
        src = blocks.vector_source_c(tx_data, False, 1, (offsettag,))
        alloc = digital.ofdm_carrier_allocator_cvc(fft_len,
                                                   occupied_carriers,
                                                   pilot_carriers,
                                                   pilot_symbols, (),
                                                   self.tsb_key)
        tx_ifft = fft.fft_vcc(fft_len, False, (1.0 / fft_len,) * fft_len, True)
        oscillator = analog.sig_source_c(
            1.0, analog.GR_COS_WAVE, freq_offset, 1.0)
        mixer = blocks.multiply_cc()
        rx_fft = fft.fft_vcc(fft_len, True, (), True)
        sink2 = blocks.tsb_vector_sink_c(vlen=fft_len, tsb_key=self.tsb_key)
        self.tb.connect(rx_fft, sink2)
        serializer = digital.ofdm_serializer_vcc(
            alloc, "", 0, "ofdm_sync_carr_offset", True
        )
        sink = blocks.tsb_vector_sink_c(tsb_key=self.tsb_key)
        self.tb.connect(
            src,
            blocks.stream_to_tagged_stream(
                gr.sizeof_gr_complex,
                1,
                len(tx_data),
                self.tsb_key),
            alloc,
            tx_ifft,
            blocks.vector_to_stream(
                gr.sizeof_gr_complex,
                fft_len),
            (mixer,
             0),
            blocks.stream_to_vector(
                gr.sizeof_gr_complex,
                fft_len),
            rx_fft,
            serializer,
            sink)
        self.tb.connect(oscillator, (mixer, 1))
        self.tb.run()
        self.assertComplexTuplesAlmostEqual(
            sink.data()[0][-len(occupied_carriers[0]):], tx_data, places=4)

    def test_005_packet_len_tag(self):
        """ Standard test """
        fft_len = 16
        tx_symbols = list(range(1, 16))
        tx_symbols = (0, 1, 1j, 2, 3, 0, 0, 0, 0, 0, 0, 4, 5, 2j, 6, 0,
                      0, 7, 8, 3j, 9, 0, 0, 0, 0, 0, 0, 10, 4j, 11, 12, 0,
                      0, 13, 1j, 14, 15, 0, 0, 0, 0, 0, 0, 0, 0, 2j, 0, 0)
        expected_result = list(range(1, 16))
        occupied_carriers = ((1, 3, 4, 11, 12, 14), (1, 2, 4, 11, 13, 14),)
        n_syms = len(tx_symbols) // fft_len
        packet_len_tsb_key = "packet_len"
        tag2 = gr.tag_t()
        tag2.offset = 0
        tag2.key = pmt.string_to_symbol("packet_len")
        tag2.value = pmt.from_long(len(expected_result))
        src = blocks.vector_source_c(tx_symbols, False, fft_len, (tag2,))
        serializer = digital.ofdm_serializer_vcc(
            fft_len, occupied_carriers, self.tsb_key, packet_len_tsb_key, 0, "", False)
        sink = blocks.tsb_vector_sink_c(tsb_key=packet_len_tsb_key)
        self.tb.connect(
            src,
            blocks.stream_to_tagged_stream(
                gr.sizeof_gr_complex,
                fft_len,
                n_syms,
                self.tsb_key),
            serializer,
            sink)
        self.tb.run()
        self.assertEqual(sink.data()[0], expected_result)

    def test_099(self):
        """ Make sure it fails if it should """
        fft_len = 16
        occupied_carriers = ((1, 3, 4, 11, 12, 112),)  # Something invalid
        #self.assertRaises(TypeError, digital.ofdm_serializer_vcc, fft_len, occupied_carriers, self.tsb_key)
        # pybind11 raises ValueError instead of TypeError
        self.assertRaises(
            ValueError,
            digital.ofdm_serializer_vcc,
            fft_len,
            occupied_carriers,
            self.tsb_key)


if __name__ == '__main__':
    gr_unittest.run(qa_ofdm_serializer_vcc)
